# encoding: utf-8

"""
Objects related to system font file lookup.
"""

from __future__ import absolute_import, print_function

import os
import sys

from struct import calcsize, unpack_from

from ..util import lazyproperty


class FontFiles(object):
    """
    A class-based singleton serving as a lazy cache for system font details.
    """

    _font_files = None

    @classmethod
    def find(cls, family_name, is_bold, is_italic):
        """
        Return the absolute path to the installed OpenType font having
        *family_name* and the styles *is_bold* and *is_italic*.
        """
        if cls._font_files is None:
            cls._font_files = cls._installed_fonts()
        return cls._font_files[(family_name, is_bold, is_italic)]

    @classmethod
    def _installed_fonts(cls):
        """
        Return a dict mapping a font descriptor to its font file path,
        containing all the font files resident on the current machine. The
        font descriptor is a (family_name, is_bold, is_italic) 3-tuple.
        """
        fonts = {}
        for d in cls._font_directories():
            for key, path in cls._iter_font_files_in(d):
                fonts[key] = path
        return fonts

    @classmethod
    def _font_directories(cls):
        """
        Return a sequence of directory paths likely to contain fonts on the
        current platform.
        """
        if sys.platform.startswith("darwin"):
            return cls._os_x_font_directories()
        if sys.platform.startswith("win32"):
            return cls._windows_font_directories()
        if sys.platform.startswith('linux'):
            return cls._linux_font_directories()
        raise OSError("unsupported operating system")

    @classmethod
    def _iter_font_files_in(cls, directory):
        """
        Generate the OpenType font files found in and under *directory*. Each
        item is a key/value pair. The key is a (family_name, is_bold,
        is_italic) 3-tuple, like ('Arial', True, False), and the value is the
        absolute path to the font file.
        """
        for root, dirs, files in os.walk(directory):
            for filename in files:
                file_ext = os.path.splitext(filename)[1]
                if file_ext.lower() not in (".otf", ".ttf"):
                    continue
                path = os.path.abspath(os.path.join(root, filename))
                with _Font.open(path) as f:
                    yield ((f.family_name, f.is_bold, f.is_italic), path)

    @classmethod
    def _os_x_font_directories(cls):
        """
        Return a sequence of directory paths on a Mac in which fonts are
        likely to be located.
        """
        os_x_font_dirs = [
            "/Library/Fonts",
            "/Network/Library/Fonts",
            "/System/Library/Fonts",
        ]
        home = os.environ.get("HOME")
        if home is not None:
            os_x_font_dirs.extend(
                [os.path.join(home, "Library", "Fonts"), os.path.join(home, ".fonts")]
            )
        return os_x_font_dirs

    @classmethod
    def _windows_font_directories(cls):
        """
        Return a sequence of directory paths on Windows in which fonts are
        likely to be located.
        """
        return [r"C:\Windows\Fonts"]

    @classmethod
    def _linux_font_directories(cls):
        """
        Return a sequence of directory paths on Linux in which fonts are
        likely to be located.
        """
        linux_font_dirs = [
            '/usr/share/fonts',
            '/usr/local/share/fonts',
        ]
        home = os.environ.get('HOME')
        if home is not None:
            linux_font_dirs.extend([
                os.path.join(home, '.fonts')
            ])
        return linux_font_dirs

class _Font(object):
    """
    A wrapper around an OTF/TTF font file stream that knows how to parse it
    for its name and style characteristics, e.g. bold and italic.
    """

    def __init__(self, stream):
        self._stream = stream

    def __enter__(self):
        return self

    def __exit__(self, exception_type, exception_value, exception_tb):
        self._stream.close()

    @property
    def is_bold(self):
        """
        |True| if this font is marked as a bold style of its font family.
        """
        try:
            return self._tables["head"].is_bold
        except KeyError:
            # some files don't have a head table
            return False

    @property
    def is_italic(self):
        """
        |True| if this font is marked as an italic style of its font family.
        """
        try:
            return self._tables["head"].is_italic
        except KeyError:
            # some files don't have a head table
            return False

    @classmethod
    def open(cls, font_file_path):
        """
        Return a |_Font| instance loaded from *font_file_path*.
        """
        return cls(_Stream.open(font_file_path))

    @property
    def family_name(self):
        """
        The name of the typeface family for this font, e.g. 'Arial'. The full
        typeface name includes optional style names, such as 'Regular' or
        'Bold Italic'. This attribute is only the common base name shared by
        all fonts in the family.
        """
        return self._tables["name"].family_name

    @lazyproperty
    def _fields(self):
        """
        A 5-tuple containing the fields read from the font file header, also
        known as the offset table.
        """
        # sfnt_version, tbl_count, search_range, entry_selector, range_shift
        return self._stream.read_fields(">4sHHHH", 0)

    def _iter_table_records(self):
        """
        Generate a (tag, offset, length) 3-tuple for each of the tables in
        this font file.
        """
        count = self._table_count
        bufr = self._stream.read(offset=12, length=count * 16)
        tmpl = ">4sLLL"
        for i in range(count):
            offset = i * 16
            tag, checksum, off, len_ = unpack_from(tmpl, bufr, offset)
            yield tag.decode("utf-8"), off, len_

    @lazyproperty
    def _tables(self):
        """
        A mapping of OpenType table tag, e.g. 'name', to a table object
        providing access to the contents of that table.
        """
        return dict(
            (tag, _TableFactory(tag, self._stream, off, len_))
            for tag, off, len_ in self._iter_table_records()
        )

    @property
    def _table_count(self):
        """
        The number of tables in this OpenType font file.
        """
        return self._fields[1]


class _Stream(object):
    """
    A thin wrapper around a file that facilitates reading C-struct values
    from a binary file.
    """

    def __init__(self, file):
        self._file = file

    @classmethod
    def open(cls, path):
        """
        Return a |_Stream| providing binary access to the contents of the
        file at *path*.
        """
        return cls(open(path, "rb"))

    def close(self):
        """
        Close the wrapped file. Using the stream after closing raises an
        exception.
        """
        self._file.close()

    def read(self, offset, length):
        """
        Return *length* bytes from this stream starting at *offset*.
        """
        self._file.seek(offset)
        return self._file.read(length)

    def read_fields(self, template, offset=0):
        """
        Return a tuple containing the C-struct fields in this stream
        specified by *template* and starting at *offset*.
        """
        self._file.seek(offset)
        bufr = self._file.read(calcsize(template))
        return unpack_from(template, bufr)


class _BaseTable(object):
    """
    Base class for OpenType font file table objects.
    """

    def __init__(self, tag, stream, offset, length):
        self._tag = tag
        self._stream = stream
        self._offset = offset
        self._length = length


class _HeadTable(_BaseTable):
    """
    OpenType font table having the tag 'head' and containing certain header
    information for the font, including its bold and/or italic style.
    """

    def __init__(self, tag, stream, offset, length):
        super(_HeadTable, self).__init__(tag, stream, offset, length)

    @property
    def is_bold(self):
        """
        |True| if this font is marked as having emboldened characters.
        """
        return bool(self._macStyle & 1)

    @property
    def is_italic(self):
        """
        |True| if this font is marked as having italicized characters.
        """
        return bool(self._macStyle & 2)

    @lazyproperty
    def _fields(self):
        """
        A 17-tuple containing the fields in this table.
        """
        return self._stream.read_fields(">4s4sLLHHqqhhhhHHHHH", self._offset)

    @property
    def _macStyle(self):
        """
        The unsigned short value of the 'macStyle' field in this head table.
        """
        return self._fields[12]


class _NameTable(_BaseTable):
    """
    An OpenType font table having the tag 'name' and containing the
    name-related strings for the font.
    """

    def __init__(self, tag, stream, offset, length):
        super(_NameTable, self).__init__(tag, stream, offset, length)

    @property
    def family_name(self):
        """
        The name of the typeface family for this font, e.g. 'Arial'.
        """

        def find_first(dict_, keys, default=None):
            for key in keys:
                value = dict_.get(key)
                if value is not None:
                    return value
            return default

        # keys for Unicode, Mac, and Windows family name, respectively
        return find_first(self._names, ((0, 1), (1, 1), (3, 1)))

    @staticmethod
    def _decode_name(raw_name, platform_id, encoding_id):
        """
        Return the unicode name decoded from *raw_name* using the encoding
        implied by the combination of *platform_id* and *encoding_id*.
        """
        if platform_id == 1:
            # reject non-Roman Mac font names
            if encoding_id != 0:
                return None
            return raw_name.decode("mac-roman")
        elif platform_id in (0, 3):
            return raw_name.decode("utf-16-be")
        else:
            return None

    def _iter_names(self):
        """
        Generate a key/value pair for each name in this table. The key is a
        (platform_id, name_id) 2-tuple and the value is the unicode text
        corresponding to that key.
        """
        table_format, count, strings_offset = self._table_header
        table_bytes = self._table_bytes

        for idx in range(count):
            platform_id, name_id, name = self._read_name(
                table_bytes, idx, strings_offset
            )
            if name is None:
                continue
            yield ((platform_id, name_id), name)

    @staticmethod
    def _name_header(bufr, idx):
        """
        The (platform_id, encoding_id, language_id, name_id, length,
        name_str_offset) 6-tuple encoded in each name record C-struct.
        """
        name_hdr_offset = 6 + idx * 12
        return unpack_from(">HHHHHH", bufr, name_hdr_offset)

    @staticmethod
    def _raw_name_string(bufr, strings_offset, str_offset, length):
        """
        Return the *length* bytes comprising the encoded string in *bufr* at
        *str_offset* in the strings area beginning at *strings_offset*.
        """
        offset = strings_offset + str_offset
        tmpl = "%ds" % length
        return unpack_from(tmpl, bufr, offset)[0]

    def _read_name(self, bufr, idx, strings_offset):
        """
        Return a (platform_id, name_id, name) 3-tuple like (0, 1, 'Arial')
        for the name at *idx* position in *bufr*. *strings_offset* is the
        index into *bufr* where actual name strings begin. The returned name
        is a unicode string.
        """
        platform_id, enc_id, lang_id, name_id, length, str_offset = self._name_header(
            bufr, idx
        )
        name = self._read_name_text(
            bufr, platform_id, enc_id, strings_offset, str_offset, length
        )
        return platform_id, name_id, name

    def _read_name_text(
        self, bufr, platform_id, encoding_id, strings_offset, name_str_offset, length
    ):
        """
        Return the unicode name string at *name_str_offset* or |None| if
        decoding its format is not supported.
        """
        raw_name = self._raw_name_string(bufr, strings_offset, name_str_offset, length)
        return self._decode_name(raw_name, platform_id, encoding_id)

    @lazyproperty
    def _table_bytes(self):
        """
        The binary contents of this name table.
        """
        return self._stream.read(self._offset, self._length)

    @property
    def _table_header(self):
        """
        The (table_format, name_count, strings_offset) 3-tuple contained
        in the header of this table.
        """
        return unpack_from(">HHH", self._table_bytes)

    @lazyproperty
    def _names(self):
        """
        A mapping of (platform_id, name_id) keys to string names for this
        font.
        """
        return dict(self._iter_names())


def _TableFactory(tag, stream, offset, length):
    """
    Return an instance of |Table| appropriate to *tag*, loaded from
    *font_file* with content of *length* starting at *offset*.
    """
    TableClass = {"head": _HeadTable, "name": _NameTable}.get(tag, _BaseTable)
    return TableClass(tag, stream, offset, length)



# encoding: utf-8

"""
Directly exposed API classes, Presentation for now. Provides some syntactic
sugar for interacting with the pptx.presentation.Package graph and also
provides some insulation so not so many classes in the other modules need to
be named as internal (leading underscore).
"""

from __future__ import absolute_import, division, print_function, unicode_literals

import os

from .opc.constants import CONTENT_TYPE as CT
from .package import Package


def Presentation(pptx=None):
    """
    Return a |Presentation| object loaded from *pptx*, where *pptx* can be
    either a path to a ``.pptx`` file (a string) or a file-like object. If
    *pptx* is missing or ``None``, the built-in default presentation
    "template" is loaded.
    """
    if pptx is None:
        pptx = _default_pptx_path()

    presentation_part = Package.open(pptx).main_document_part

    if not _is_pptx_package(presentation_part):
        tmpl = "file '%s' is not a PowerPoint file, content type is '%s'"
        raise ValueError(tmpl % (pptx, presentation_part.content_type))

    return presentation_part.presentation


def _default_pptx_path():
    """
    Return the path to the built-in default .pptx package.
    """
    _thisdir = os.path.split(__file__)[0]
    return os.path.join(_thisdir, "templates", "default.pptx")


def _is_pptx_package(prs_part):
    """
    Return |True| if *prs_part* is a valid main document part, |False|
    otherwise.
    """
    valid_content_types = (CT.PML_PRESENTATION_MAIN, CT.PML_PRES_MACRO_MAIN)
    return prs_part.content_type in valid_content_types



# encoding: utf-8

"""
Plot-related objects. A plot is known as a chart group in the MS API. A chart
can have more than one plot overlayed on each other, such as a line plot
layered over a bar plot.
"""

from __future__ import absolute_import, print_function, unicode_literals

from .category import Categories
from .datalabel import DataLabels
from ..enum.chart import XL_CHART_TYPE as XL
from ..oxml.ns import qn
from ..oxml.simpletypes import ST_BarDir, ST_Grouping
from .series import SeriesCollection
from ..util import lazyproperty


class _BasePlot(object):
    """
    A distinct plot that appears in the plot area of a chart. A chart may
    have more than one plot, in which case they appear as superimposed
    layers, such as a line plot appearing on top of a bar chart.
    """

    def __init__(self, xChart, chart):
        super(_BasePlot, self).__init__()
        self._element = xChart
        self._chart = chart

    @lazyproperty
    def categories(self):
        """
        Returns a |category.Categories| sequence object containing
        a |category.Category| object for each of the category labels
        associated with this plot. The |category.Category| class derives from
        ``str``, so the returned value can be treated as a simple sequence of
        strings for the common case where all you need is the labels in the
        order they appear on the chart. |category.Categories| provides
        additional properties for dealing with hierarchical categories when
        required.
        """
        return Categories(self._element)

    @property
    def chart(self):
        """
        The |Chart| object containing this plot.
        """
        return self._chart

    @property
    def data_labels(self):
        """
        |DataLabels| instance providing properties and methods on the
        collection of data labels associated with this plot.
        """
        dLbls = self._element.dLbls
        if dLbls is None:
            raise ValueError(
                "plot has no data labels, set has_data_labels = True first"
            )
        return DataLabels(dLbls)

    @property
    def has_data_labels(self):
        """
        Read/write boolean, |True| if the series has data labels. Assigning
        |True| causes data labels to be added to the plot. Assigning False
        removes any existing data labels.
        """
        return self._element.dLbls is not None

    @has_data_labels.setter
    def has_data_labels(self, value):
        """
        Add, remove, or leave alone the ``<c:dLbls>`` child element depending
        on current state and assigned *value*. If *value* is |True| and no
        ``<c:dLbls>`` element is present, a new default element is added with
        default child elements and settings. When |False|, any existing dLbls
        element is removed.
        """
        if bool(value) is False:
            self._element._remove_dLbls()
        else:
            if self._element.dLbls is None:
                dLbls = self._element._add_dLbls()
                dLbls.showVal.val = True

    @lazyproperty
    def series(self):
        """
        A sequence of |Series| objects representing the series in this plot,
        in the order they appear in the plot.
        """
        return SeriesCollection(self._element)

    @property
    def vary_by_categories(self):
        """
        Read/write boolean value specifying whether to use a different color
        for each of the points in this plot. Only effective when there is
        a single series; PowerPoint automatically varies color by series when
        more than one series is present.
        """
        varyColors = self._element.varyColors
        if varyColors is None:
            return True
        return varyColors.val

    @vary_by_categories.setter
    def vary_by_categories(self, value):
        self._element.get_or_add_varyColors().val = bool(value)


class AreaPlot(_BasePlot):
    """
    An area plot.
    """


class Area3DPlot(_BasePlot):
    """
    A 3-dimensional area plot.
    """


class BarPlot(_BasePlot):
    """
    A bar chart-style plot.
    """

    @property
    def gap_width(self):
        """
        Width of gap between bar(s) of each category, as an integer
        percentage of the bar width. The default value for a new bar chart is
        150, representing 150% or 1.5 times the width of a single bar.
        """
        gapWidth = self._element.gapWidth
        if gapWidth is None:
            return 150
        return gapWidth.val

    @gap_width.setter
    def gap_width(self, value):
        gapWidth = self._element.get_or_add_gapWidth()
        gapWidth.val = value

    @property
    def overlap(self):
        """
        Read/write int value in range -100..100 specifying a percentage of
        the bar width by which to overlap adjacent bars in a multi-series bar
        chart. Default is 0. A setting of -100 creates a gap of a full bar
        width and a setting of 100 causes all the bars in a category to be
        superimposed. A stacked bar plot has overlap of 100 by default.
        """
        overlap = self._element.overlap
        if overlap is None:
            return 0
        return overlap.val

    @overlap.setter
    def overlap(self, value):
        """
        Set the value of the ``<c:overlap>`` child element to *int_value*,
        or remove the overlap element if *int_value* is 0.
        """
        if value == 0:
            self._element._remove_overlap()
            return
        self._element.get_or_add_overlap().val = value


class BubblePlot(_BasePlot):
    """
    A bubble chart plot.
    """

    @property
    def bubble_scale(self):
        """
        An integer between 0 and 300 inclusive indicating the percentage of
        the default size at which bubbles should be displayed. Assigning
        |None| produces the same behavior as assigning `100`.
        """
        bubbleScale = self._element.bubbleScale
        if bubbleScale is None:
            return 100
        return bubbleScale.val

    @bubble_scale.setter
    def bubble_scale(self, value):
        bubbleChart = self._element
        bubbleChart._remove_bubbleScale()
        if value is None:
            return
        bubbleScale = bubbleChart._add_bubbleScale()
        bubbleScale.val = value


class DoughnutPlot(_BasePlot):
    """
    An doughnut plot.
    """


class LinePlot(_BasePlot):
    """
    A line chart-style plot.
    """


class PiePlot(_BasePlot):
    """
    A pie chart-style plot.
    """


class RadarPlot(_BasePlot):
    """
    A radar-style plot.
    """


class XyPlot(_BasePlot):
    """
    An XY (scatter) plot.
    """


def PlotFactory(xChart, chart):
    """
    Return an instance of the appropriate subclass of _BasePlot based on the
    tagname of *xChart*.
    """
    try:
        PlotCls = {
            qn("c:areaChart"): AreaPlot,
            qn("c:area3DChart"): Area3DPlot,
            qn("c:barChart"): BarPlot,
            qn("c:bubbleChart"): BubblePlot,
            qn("c:doughnutChart"): DoughnutPlot,
            qn("c:lineChart"): LinePlot,
            qn("c:pieChart"): PiePlot,
            qn("c:radarChart"): RadarPlot,
            qn("c:scatterChart"): XyPlot,
        }[xChart.tag]
    except KeyError:
        raise ValueError("unsupported plot type %s" % xChart.tag)

    return PlotCls(xChart, chart)


class PlotTypeInspector(object):
    """
    "One-shot" service object that knows how to identify the type of a plot
    as a member of the XL_CHART_TYPE enumeration.
    """

    @classmethod
    def chart_type(cls, plot):
        """
        Return the member of :ref:`XlChartType` that corresponds to the chart
        type of *plot*.
        """
        try:
            chart_type_method = {
                "AreaPlot": cls._differentiate_area_chart_type,
                "Area3DPlot": cls._differentiate_area_3d_chart_type,
                "BarPlot": cls._differentiate_bar_chart_type,
                "BubblePlot": cls._differentiate_bubble_chart_type,
                "DoughnutPlot": cls._differentiate_doughnut_chart_type,
                "LinePlot": cls._differentiate_line_chart_type,
                "PiePlot": cls._differentiate_pie_chart_type,
                "RadarPlot": cls._differentiate_radar_chart_type,
                "XyPlot": cls._differentiate_xy_chart_type,
            }[plot.__class__.__name__]
        except KeyError:
            raise NotImplementedError(
                "chart_type() not implemented for %s" % plot.__class__.__name__
            )
        return chart_type_method(plot)

    @classmethod
    def _differentiate_area_3d_chart_type(cls, plot):
        return {
            ST_Grouping.STANDARD: XL.THREE_D_AREA,
            ST_Grouping.STACKED: XL.THREE_D_AREA_STACKED,
            ST_Grouping.PERCENT_STACKED: XL.THREE_D_AREA_STACKED_100,
        }[plot._element.grouping_val]

    @classmethod
    def _differentiate_area_chart_type(cls, plot):
        return {
            ST_Grouping.STANDARD: XL.AREA,
            ST_Grouping.STACKED: XL.AREA_STACKED,
            ST_Grouping.PERCENT_STACKED: XL.AREA_STACKED_100,
        }[plot._element.grouping_val]

    @classmethod
    def _differentiate_bar_chart_type(cls, plot):
        barChart = plot._element
        if barChart.barDir.val == ST_BarDir.BAR:
            return {
                ST_Grouping.CLUSTERED: XL.BAR_CLUSTERED,
                ST_Grouping.STACKED: XL.BAR_STACKED,
                ST_Grouping.PERCENT_STACKED: XL.BAR_STACKED_100,
            }[barChart.grouping_val]
        if barChart.barDir.val == ST_BarDir.COL:
            return {
                ST_Grouping.CLUSTERED: XL.COLUMN_CLUSTERED,
                ST_Grouping.STACKED: XL.COLUMN_STACKED,
                ST_Grouping.PERCENT_STACKED: XL.COLUMN_STACKED_100,
            }[barChart.grouping_val]
        raise ValueError("invalid barChart.barDir value '%s'" % barChart.barDir.val)

    @classmethod
    def _differentiate_bubble_chart_type(cls, plot):
        def first_bubble3D(bubbleChart):
            results = bubbleChart.xpath("c:ser/c:bubble3D")
            return results[0] if results else None

        bubbleChart = plot._element
        bubble3D = first_bubble3D(bubbleChart)

        if bubble3D is None:
            return XL.BUBBLE
        if bubble3D.val:
            return XL.BUBBLE_THREE_D_EFFECT
        return XL.BUBBLE

    @classmethod
    def _differentiate_doughnut_chart_type(cls, plot):
        doughnutChart = plot._element
        explosion = doughnutChart.xpath("./c:ser/c:explosion")
        return XL.DOUGHNUT_EXPLODED if explosion else XL.DOUGHNUT

    @classmethod
    def _differentiate_line_chart_type(cls, plot):
        lineChart = plot._element

        def has_line_markers():
            matches = lineChart.xpath('c:ser/c:marker/c:symbol[@val="none"]')
            if matches:
                return False
            return True

        if has_line_markers():
            return {
                ST_Grouping.STANDARD: XL.LINE_MARKERS,
                ST_Grouping.STACKED: XL.LINE_MARKERS_STACKED,
                ST_Grouping.PERCENT_STACKED: XL.LINE_MARKERS_STACKED_100,
            }[plot._element.grouping_val]
        else:
            return {
                ST_Grouping.STANDARD: XL.LINE,
                ST_Grouping.STACKED: XL.LINE_STACKED,
                ST_Grouping.PERCENT_STACKED: XL.LINE_STACKED_100,
            }[plot._element.grouping_val]

    @classmethod
    def _differentiate_pie_chart_type(cls, plot):
        pieChart = plot._element
        explosion = pieChart.xpath("./c:ser/c:explosion")
        return XL.PIE_EXPLODED if explosion else XL.PIE

    @classmethod
    def _differentiate_radar_chart_type(cls, plot):
        radarChart = plot._element
        radar_style = radarChart.xpath("c:radarStyle")[0].get("val")

        def noMarkers():
            matches = radarChart.xpath("c:ser/c:marker/c:symbol")
            if matches and matches[0].get("val") == "none":
                return True
            return False

        if radar_style is None:
            return XL.RADAR
        if radar_style == "filled":
            return XL.RADAR_FILLED
        if noMarkers():
            return XL.RADAR
        return XL.RADAR_MARKERS

    @classmethod
    def _differentiate_xy_chart_type(cls, plot):
        scatterChart = plot._element

        def noLine():
            return bool(scatterChart.xpath("c:ser/c:spPr/a:ln/a:noFill"))

        def noMarkers():
            symbols = scatterChart.xpath("c:ser/c:marker/c:symbol")
            if symbols and symbols[0].get("val") == "none":
                return True
            return False

        scatter_style = scatterChart.xpath("c:scatterStyle")[0].get("val")

        if scatter_style == "lineMarker":
            if noLine():
                return XL.XY_SCATTER
            if noMarkers():
                return XL.XY_SCATTER_LINES_NO_MARKERS
            return XL.XY_SCATTER_LINES

        if scatter_style == "smoothMarker":
            if noMarkers():
                return XL.XY_SCATTER_SMOOTH_NO_MARKERS
            return XL.XY_SCATTER_SMOOTH

        return XL.XY_SCATTER



# encoding: utf-8

"""Gherkin step implementations for chart plot features."""

from __future__ import absolute_import, division, print_function, unicode_literals

from ast import literal_eval

from behave import given, then, when

from pptx import Presentation
from pptx.dml.color import RGBColor
from pptx.enum.chart import XL_MARKER_STYLE
from pptx.enum.dml import MSO_FILL_TYPE, MSO_THEME_COLOR

from helpers import test_pptx


# given ===================================================


@given("a BarSeries object having {fill_type} fill as series")
def given_a_BarSeries_object_having_fill_type_as_series(context, fill_type):
    series_idx = {"Automatic": 0, "No Fill": 1, "Orange": 2, "Accent 1": 3}[fill_type]
    prs = Presentation(test_pptx("cht-series"))
    plot = prs.slides[2].shapes[0].chart.plots[0]
    context.series = plot.series[series_idx]


@given("a BarSeries object having invert_if_negative of {setting} as series")
def given_a_bar_series_having_invert_if_negative_setting(context, setting):
    series_idx = {"no explicit setting": 0, "True": 1, "False": 2}[setting]
    prs = Presentation(test_pptx("cht-series"))
    plot = prs.slides[2].shapes[0].chart.plots[0]
    context.series = plot.series[series_idx]


@given("a BarSeries object having values {values} as series")
def given_a_bar_series_having_values_as_series(context, values):
    prs = Presentation(test_pptx("cht-series"))
    series_idx = {"1.2, 2.3, 3.4": 0, "4.5, None, 6.7": 1}[values]
    context.series = prs.slides[3].shapes[0].chart.plots[0].series[series_idx]


@given("a BarSeries object having {width} line as series")
def given_a_bar_series_having_width_line_as_series(context, width):
    series_idx = {"no": 0, "1 point": 1}[width]
    prs = Presentation(test_pptx("cht-series"))
    plot = prs.slides[2].shapes[0].chart.plots[0]
    context.series = plot.series[series_idx]


@given("a marker")
def given_a_marker(context):
    prs = Presentation(test_pptx("cht-marker-props"))
    series = prs.slides[0].shapes[0].chart.series[0]
    context.marker = series.marker


@given("a marker having size of {case}")
def given_a_marker_having_size_of_case(context, case):
    series_idx = {"no explicit value": 0, "24 points": 1, "36 points": 2}[case]
    prs = Presentation(test_pptx("cht-marker-props"))
    series = prs.slides[0].shapes[0].chart.series[series_idx]
    context.marker = series.marker


@given("a marker having style of {case}")
def given_a_marker_having_style_of_case(context, case):
    series_idx = {"no explicit value": 0, "circle": 1, "triangle": 2}[case]
    prs = Presentation(test_pptx("cht-marker-props"))
    series = prs.slides[0].shapes[0].chart.series[series_idx]
    context.marker = series.marker


@given("a point")
def given_a_point(context):
    prs = Presentation(test_pptx("cht-point-props"))
    chart = prs.slides[0].shapes[0].chart
    context.point = chart.plots[0].series[0].points[0]


@given("a {points_type} object containing 3 points")
def given_a_points_type_object_containing_3_points(context, points_type):
    slide_idx = {"XyPoints": 0, "BubblePoints": 1, "CategoryPoints": 2}[points_type]
    prs = Presentation(test_pptx("cht-point-access"))
    series = prs.slides[slide_idx].shapes[0].chart.plots[0].series[0]
    context.points = series.points


@given("a series")
def given_a_series(context):
    prs = Presentation(test_pptx("cht-series"))
    context.series = prs.slides[3].shapes[0].chart.plots[0].series[0]


@given("a {prefix}Series object as series")
def given_a_series_of_type_series_type(context, prefix):
    slide_idx = {
        "Area": 8,
        "Bar": 3,
        "Bubble": 5,
        "Category": 3,
        "Doughnut": 9,
        "Line": 6,
        "Pie": 10,
        "Radar": 7,
        "Xy": 4,
    }[prefix]
    prs = Presentation(test_pptx("cht-series"))
    context.series = prs.slides[slide_idx].shapes[0].chart.plots[0].series[0]


@given("a SeriesCollection object for a plot having {n} series")
def given_a_SeriesCollection_object_for_a_plot_having_n_series(context, n):
    prs = Presentation(test_pptx("cht-series"))
    plot = prs.slides[0].shapes[0].chart.plots[0]
    context.series_collection = plot.series
    context.series_count = int(n)


@given("a SeriesCollection object for a {type_} chart having {n} series")
def given_a_SeriesCollection_for_chart_having_n_series(context, type_, n):
    slide_idx = {"single-plot": 0, "multi-plot": 1}[type_]
    prs = Presentation(test_pptx("cht-series"))
    context.series_collection = prs.slides[slide_idx].shapes[0].chart.series
    context.series_count = int(n)


# when ====================================================


@when("I add a series with number format {strval}")
def when_I_add_a_series_with_number_format(context, strval):
    chart_data = context.chart_data
    params = {"name": "Series Foo"}
    if strval != "None":
        params["number_format"] = int(strval)
    context.series_data = chart_data.add_series(**params)


@when("I assign {value} to marker.size")
def when_I_assign_value_to_marker_size(context, value):
    new_value = None if value == "None" else int(value)
    context.marker.size = new_value


@when("I assign {value} to marker.style")
def when_I_assign_value_to_marker_style(context, value):
    new_value = None if value == "None" else getattr(XL_MARKER_STYLE, value)
    context.marker.style = new_value


@when("I assign {value} to series.invert_if_negative")
def when_I_assign_value_to_series_invert_if_negative(context, value):
    new_value = {"True": True, "False": False}[value]
    context.series.invert_if_negative = new_value


# then ====================================================


@then("data_point.number_format is {value_str}")
def then_data_point_number_format_is(context, value_str):
    data_point = context.data_point
    number_format = value_str if value_str == "General" else int(value_str)
    assert data_point.number_format == number_format


@then("iterating points produces 3 Point objects")
def then_iterating_points_produces_3_point_objects(context):
    points = context.points
    idx = -1
    for idx, point in enumerate(points):
        assert type(point).__name__ == "Point"
    assert idx == 2, "got %s" % idx


@then("iterating series_collection produces {count} Series objects")
def then_iterating_series_collection_produces_count_series(context, count):
    expected_idx = int(count) - 1
    idx = -1
    for idx, series in enumerate(context.series_collection):
        type_name = type(series).__name__
        assert type_name.endswith("Series"), "got %s" % type_name
    assert idx == expected_idx, "got %s" % idx


@then("len(points) is 3")
def then_len_points_is_3(context):
    points = context.points
    assert len(points) == 3


@then("len(series_collection) is {count}")
def then_len_series_collection_is_count(context, count):
    expected_len = int(count)
    actual_len = len(context.series_collection)
    assert actual_len == expected_len, "got %s" % actual_len


@then("len(series.values) is {count} for each series")
def then_len_series_values_is_count_for_each_series(context, count):
    expected_count = int(count)
    for series in context.chart.plots[0].series:
        assert len(series.values) == expected_count


@then("marker.format is a ChartFormat object")
def then_marker_format_is_a_ChartFormat_object(context):
    marker = context.marker
    assert type(marker.format).__name__ == "ChartFormat"


@then("marker.format.fill is a FillFormat object")
def then_marker_format_fill_is_a_FillFormat_object(context):
    marker = context.marker
    assert type(marker.format.fill).__name__ == "FillFormat"


@then("marker.format.line is a LineFormat object")
def then_marker_format_line_is_a_LineFormat_object(context):
    marker = context.marker
    assert type(marker.format.line).__name__ == "LineFormat"


@then("marker.size is {case}")
def then_marker_size_is_case(context, case):
    expected_value = None if case == "None" else int(case)
    marker = context.marker
    assert marker.size == expected_value, "got %s" % marker.size


@then("marker.style is {case}")
def then_marker_style_is_case(context, case):
    expected_value = None if case == "None" else getattr(XL_MARKER_STYLE, case)
    marker = context.marker
    assert marker.style == expected_value, "got %s" % marker.style


@then("point.data_label is a DataLabel object")
def then_point_data_label_is_a_DataLabel_object(context):
    point = context.point
    assert type(point.data_label).__name__ == "DataLabel"


@then("point.format is a ChartFormat object")
def then_point_format_is_a_ChartFormat_object(context):
    point = context.point
    assert type(point.format).__name__ == "ChartFormat"


@then("point.format.fill is a FillFormat object")
def then_point_format_fill_is_a_FillFormat_object(context):
    point = context.point
    assert type(point.format.fill).__name__ == "FillFormat"


@then("point.format.line is a LineFormat object")
def then_point_format_line_is_a_LineFormat_object(context):
    point = context.point
    assert type(point.format.line).__name__ == "LineFormat"


@then("point.marker is a Marker object")
def then_point_marker_is_a_Marker_object(context):
    point = context.point
    assert type(point.marker).__name__ == "Marker"


@then("points[2] is a Point object")
def then_points_2_is_a_Point_object(context):
    actual = type(context.points[2]).__name__
    assert actual == "Point", "points[2] is a %s object" % actual


@then("series_collection[2] is a Series object")
def then_series_collection_2_is_a_Series_object(context):
    type_name = type(context.series_collection[2]).__name__
    assert type_name.endswith("Series"), "got %s" % type_name


@then("series.data_labels is a DataLabels object")
def then_series_data_labels_is_a_DataLabels_object(context):
    actual = type(context.series.data_labels).__name__
    assert actual == "DataLabels", "series.data_labels is a %s object" % actual


@then("series.format.fill.fore_color.rgb is FF6600")
def then_series_format_fill_fore_color_rgb_is_FF6600(context):
    rgb_color = context.series.format.fill.fore_color.rgb
    assert rgb_color == RGBColor(0xFF, 0x66, 0x00), "got %s" % rgb_color


@then("series.format.fill.fore_color.theme_color is Accent 1")
def then_series_format_fill_fore_color_theme_color_is_Accent_1(context):
    theme_color = context.series.format.fill.fore_color.theme_color
    assert theme_color == MSO_THEME_COLOR.ACCENT_1, "got %s" % theme_color


@then("series.format.fill.type is {fill_type}")
def then_series_format_fill_type_is_type(context, fill_type):
    expected_fill_type = {
        "None": None,
        "MSO_FILL_TYPE.BACKGROUND": MSO_FILL_TYPE.BACKGROUND,
        "MSO_FILL_TYPE.SOLID": MSO_FILL_TYPE.SOLID,
    }[fill_type]
    fill_type = context.series.format.fill.type
    assert fill_type == expected_fill_type, "got %s" % fill_type


@then("series.invert_if_negative is {value}")
def then_series_invert_if_negative_is_value(context, value):
    expected_value = {"True": True, "False": False}[value]
    series = context.series
    assert series.invert_if_negative is expected_value


@then("series.format.line.width is {width}")
def then_series_format_line_width_is_width(context, width):
    expected_width = int(width)
    line_width = context.series.format.line.width
    assert line_width == expected_width, "got %s" % line_width


@then("series.format is a ChartFormat object")
def then_series_format_is_a_ChartFormat_object(context):
    actual = type(context.series.format).__name__
    assert actual == "ChartFormat", "series.format is a %s object" % actual


@then("series.marker is a Marker object")
def then_series_marker_is_a_Marker_object(context):
    actual = type(context.series.marker).__name__
    assert actual == "Marker", "series.marker is a %s object" % actual


@then("series.points is a {type_name} object")
def then_series_points_is_a_type_name_object(context, type_name):
    actual = type(context.series.points).__name__
    expected = type_name
    assert actual == expected, "series.points is a %s object" % actual


@then("series.values is {values}")
def then_series_values_is_values(context, values):
    series = context.series
    expected_values = literal_eval(values)
    assert series.values == expected_values, "got %s" % (series.values,)



# encoding: utf-8

"""
Chart part objects, including Chart and Charts
"""

from __future__ import absolute_import, print_function, unicode_literals

from ..chart.chart import Chart
from .embeddedpackage import EmbeddedXlsxPart
from ..opc.constants import CONTENT_TYPE as CT, RELATIONSHIP_TYPE as RT
from ..opc.package import XmlPart
from ..util import lazyproperty


class ChartPart(XmlPart):
    """
    A chart part; corresponds to parts having partnames matching
    ppt/charts/chart[1-9][0-9]*.xml
    """

    partname_template = "/ppt/charts/chart%d.xml"

    @classmethod
    def new(cls, chart_type, chart_data, package):
        """
        Return a new |ChartPart| instance added to *package* containing
        a chart of *chart_type* and depicting *chart_data*.
        """
        chart_blob = chart_data.xml_bytes(chart_type)
        partname = package.next_partname(cls.partname_template)
        content_type = CT.DML_CHART
        chart_part = cls.load(partname, content_type, chart_blob, package)
        xlsx_blob = chart_data.xlsx_blob
        chart_part.chart_workbook.update_from_xlsx_blob(xlsx_blob)
        return chart_part

    @lazyproperty
    def chart(self):
        """
        The |Chart| object representing the chart in this part.
        """
        return Chart(self._element, self)

    @lazyproperty
    def chart_workbook(self):
        """
        The |ChartWorkbook| object providing access to the external chart
        data in a linked or embedded Excel workbook.
        """
        return ChartWorkbook(self._element, self)


class ChartWorkbook(object):
    """
    Provides access to the external chart data in a linked or embedded Excel
    workbook.
    """

    def __init__(self, chartSpace, chart_part):
        super(ChartWorkbook, self).__init__()
        self._chartSpace = chartSpace
        self._chart_part = chart_part

    def update_from_xlsx_blob(self, xlsx_blob):
        """
        Replace the Excel spreadsheet in the related |EmbeddedXlsxPart| with
        the Excel binary in *xlsx_blob*, adding a new |EmbeddedXlsxPart| if
        there isn't one.
        """
        xlsx_part = self.xlsx_part
        if xlsx_part is None:
            self.xlsx_part = EmbeddedXlsxPart.new(xlsx_blob, self._package)
            return
        xlsx_part.blob = xlsx_blob

    @property
    def xlsx_part(self):
        """
        Return the related |EmbeddedXlsxPart| object having its rId at
        `c:chartSpace/c:externalData/@rId` or |None| if there is no
        `<c:externalData>` element.
        """
        xlsx_part_rId = self._chartSpace.xlsx_part_rId
        if xlsx_part_rId is None:
            return None
        return self._chart_part.related_parts[xlsx_part_rId]

    @xlsx_part.setter
    def xlsx_part(self, xlsx_part):
        """
        Set the related |EmbeddedXlsxPart| to *xlsx_part*. Assume one does
        not already exist.
        """
        rId = self._chart_part.relate_to(xlsx_part, RT.PACKAGE)
        externalData = self._chartSpace.get_or_add_externalData()
        externalData.rId = rId

    @property
    def _package(self):
        return self._chart_part.package



# encoding: utf-8

"""Test suite for pptx.chart.series module."""

from __future__ import absolute_import, division, print_function, unicode_literals

import pytest

from pptx.chart.datalabel import DataLabels
from pptx.chart.marker import Marker
from pptx.chart.point import BubblePoints, CategoryPoints, XyPoints
from pptx.chart.series import (
    AreaSeries,
    BarSeries,
    _BaseCategorySeries,
    _BaseSeries,
    BubbleSeries,
    LineSeries,
    _MarkerMixin,
    PieSeries,
    RadarSeries,
    SeriesCollection,
    _SeriesFactory,
    XySeries,
)
from pptx.dml.chtfmt import ChartFormat

from ..unitutil.cxml import element, xml
from ..unitutil.mock import class_mock, function_mock, instance_mock


class Describe_BaseSeries(object):
    def it_knows_its_name(self, name_fixture):
        series, expected_value = name_fixture
        assert series.name == expected_value

    def it_knows_its_position_in_the_series_sequence(self, index_fixture):
        series, expected_value = index_fixture
        assert series.index == expected_value

    def it_provides_access_to_its_format(self, format_fixture):
        series, ChartFormat_, ser, format_ = format_fixture
        format = series.format
        ChartFormat_.assert_called_once_with(ser)
        assert format is format_

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def format_fixture(self, ChartFormat_, chart_format_):
        ser = element("c:ser")
        series = _BaseSeries(ser)
        return series, ChartFormat_, ser, chart_format_

    @pytest.fixture
    def index_fixture(self):
        ser_cxml, expected_value = "c:ser/c:idx{val=42}", 42
        series = _BaseSeries(element(ser_cxml))
        return series, expected_value

    @pytest.fixture(
        params=[
            ("c:ser", ""),
            ('c:ser/c:tx/c:strRef/c:strCache/c:pt/c:v"foobar"', "foobar"),
        ]
    )
    def name_fixture(self, request):
        ser_cxml, expected_value = request.param
        series = _BaseSeries(element(ser_cxml))
        return series, expected_value

    # fixture components ---------------------------------------------

    @pytest.fixture
    def ChartFormat_(self, request, chart_format_):
        return class_mock(
            request, "pptx.chart.series.ChartFormat", return_value=chart_format_
        )

    @pytest.fixture
    def chart_format_(self, request):
        return instance_mock(request, ChartFormat)


class Describe_BaseCategorySeries(object):
    def it_is_a_BaseSeries_subclass(self, subclass_fixture):
        base_category_series = subclass_fixture
        assert isinstance(base_category_series, _BaseSeries)

    def it_provides_access_to_its_data_labels(
        self, data_labels_fixture, DataLabels_, data_labels_
    ):
        ser, expected_dLbls_xml = data_labels_fixture
        DataLabels_.return_value = data_labels_
        series = _BaseCategorySeries(ser)

        data_labels = series.data_labels

        dLbls = ser.xpath("c:dLbls")[0]
        assert dLbls.xml == expected_dLbls_xml
        DataLabels_.assert_called_once_with(dLbls)
        assert data_labels is data_labels_

    def it_provides_access_to_its_points(self, points_fixture):
        series, CategoryPoints_, ser, points_ = points_fixture
        points = series.points
        CategoryPoints_.assert_called_once_with(ser)
        assert points is points_

    def it_knows_its_values(self, values_get_fixture):
        series, expected_value = values_get_fixture
        assert series.values == expected_value

    # fixtures -------------------------------------------------------

    @pytest.fixture(
        params=[
            (
                "c:ser",
                "c:dLbls/(c:showLegendKey{val=0},c:showVal{val=0},c:showCatName{val"
                "=0},c:showSerName{val=0},c:showPercent{val=0},c:showBubbleSize{val"
                "=0},c:showLeaderLines{val=1})",
            ),
            ("c:ser/c:dLbls", "c:dLbls"),
        ]
    )
    def data_labels_fixture(self, request):
        ser_cxml, expected_dLbls_cxml = request.param
        ser = element(ser_cxml)
        expected_dLbls_xml = xml(expected_dLbls_cxml)
        return ser, expected_dLbls_xml

    @pytest.fixture
    def points_fixture(self, CategoryPoints_, points_):
        ser = element("c:ser")
        series = _BaseCategorySeries(ser)
        return series, CategoryPoints_, ser, points_

    @pytest.fixture
    def subclass_fixture(self):
        return _BaseCategorySeries(None)

    @pytest.fixture(
        params=[
            ("c:ser", ()),
            ("c:ser/c:val/c:numRef", ()),
            ("c:ser/c:val/c:numLit", ()),
            ("c:ser/c:val/c:numRef/c:numCache", ()),
            ("c:ser/c:val/c:numRef/c:numCache/c:ptCount{val=0}", ()),
            (
                'c:ser/c:val/c:numRef/c:numCache/(c:ptCount{val=1},c:pt{idx=0}/c:v"'
                '1.1")',
                (1.1,),
            ),
            (
                'c:ser/c:val/c:numRef/c:numCache/(c:ptCount{val=3},c:pt{idx=0}/c:v"'
                '1.1",c:pt{idx=2}/c:v"3.3")',
                (1.1, None, 3.3),
            ),
            (
                'c:ser/c:val/c:numLit/(c:ptCount{val=3},c:pt{idx=0}/c:v"1.1",c:pt{i'
                'dx=2}/c:v"3.3")',
                (1.1, None, 3.3),
            ),
            (
                'c:ser/c:val/c:numRef/c:numCache/(c:ptCount{val=3},c:pt{idx=2}/c:v"'
                '3.3",c:pt{idx=0}/c:v"1.1")',
                (1.1, None, 3.3),
            ),
        ]
    )
    def values_get_fixture(self, request):
        ser_cxml, expected_value = request.param
        series = _BaseCategorySeries(element(ser_cxml))
        return series, expected_value

    # fixture components ---------------------------------------------

    @pytest.fixture
    def CategoryPoints_(self, request, points_):
        return class_mock(
            request, "pptx.chart.series.CategoryPoints", return_value=points_
        )

    @pytest.fixture
    def DataLabels_(self, request):
        return class_mock(request, "pptx.chart.series.DataLabels")

    @pytest.fixture
    def data_labels_(self, request):
        return instance_mock(request, DataLabels)

    @pytest.fixture
    def points_(self, request):
        return instance_mock(request, CategoryPoints)


class Describe_MarkerMixin(object):
    def it_provides_access_to_the_series_marker(self, marker_fixture):
        series, Marker_, ser, marker_ = marker_fixture
        marker = series.marker
        Marker_.assert_called_once_with(ser)
        assert marker is marker_

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def marker_fixture(self, Marker_, marker_):
        ser = element("c:ser")
        series = LineSeries(ser)
        return series, Marker_, ser, marker_

    # fixture components ---------------------------------------------

    @pytest.fixture
    def Marker_(self, request, marker_):
        return class_mock(request, "pptx.chart.series.Marker", return_value=marker_)

    @pytest.fixture
    def marker_(self, request):
        return instance_mock(request, Marker)


class DescribeAreaSeries(object):
    def it_is_a_BaseCategorySeries_subclass(self, subclass_fixture):
        area_series = subclass_fixture
        assert isinstance(area_series, _BaseCategorySeries)

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def subclass_fixture(self):
        return AreaSeries(None)


class DescribeBarSeries(object):
    def it_is_a_BaseCategorySeries_subclass(self, subclass_fixture):
        bar_series = subclass_fixture
        assert isinstance(bar_series, _BaseCategorySeries)

    def it_knows_whether_it_should_invert_if_negative(
        self, invert_if_negative_get_fixture
    ):
        bar_series, expected_value = invert_if_negative_get_fixture
        assert bar_series.invert_if_negative == expected_value

    def it_can_change_whether_it_inverts_if_negative(
        self, invert_if_negative_set_fixture
    ):
        bar_series, new_value, expected_xml = invert_if_negative_set_fixture
        bar_series.invert_if_negative = new_value
        assert bar_series._element.xml == expected_xml

    # fixtures -------------------------------------------------------

    @pytest.fixture(
        params=[
            ("c:ser", True),
            ("c:ser/c:invertIfNegative", True),
            ("c:ser/c:invertIfNegative{val=1}", True),
            ("c:ser/c:invertIfNegative{val=0}", False),
        ]
    )
    def invert_if_negative_get_fixture(self, request):
        ser_cxml, expected_value = request.param
        bar_series = BarSeries(element(ser_cxml))
        return bar_series, expected_value

    @pytest.fixture(
        params=[
            ("c:ser/c:order", True, "c:ser/(c:order,c:invertIfNegative{val=1})"),
            ("c:ser/c:order", False, "c:ser/(c:order,c:invertIfNegative{val=0})"),
            (
                "c:ser/(c:spPr,c:invertIfNegative{val=0})",
                True,
                "c:ser/(c:spPr,c:invertIfNegative{val=1})",
            ),
            (
                "c:ser/(c:tx,c:invertIfNegative{val=1})",
                False,
                "c:ser/(c:tx,c:invertIfNegative{val=0})",
            ),
        ]
    )
    def invert_if_negative_set_fixture(self, request):
        ser_cxml, new_value, expected_ser_cxml = request.param
        bar_series = BarSeries(element(ser_cxml))
        expected_xml = xml(expected_ser_cxml)
        return bar_series, new_value, expected_xml

    @pytest.fixture
    def subclass_fixture(self):
        return BarSeries(None)


class Describe_BubbleSeries(object):
    def it_provides_access_to_its_points(self, points_fixture):
        series, BubblePoints_, ser, points_ = points_fixture
        points = series.points
        BubblePoints_.assert_called_once_with(ser)
        assert points is points_

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def points_fixture(self, BubblePoints_, points_):
        ser = element("c:ser")
        series = BubbleSeries(ser)
        return series, BubblePoints_, ser, points_

    # fixture components ---------------------------------------------

    @pytest.fixture
    def BubblePoints_(self, request, points_):
        return class_mock(
            request, "pptx.chart.series.BubblePoints", return_value=points_
        )

    @pytest.fixture
    def points_(self, request):
        return instance_mock(request, BubblePoints)


class DescribeLineSeries(object):
    def it_is_a_BaseCategorySeries_subclass(self, subclass_fixture):
        line_series = subclass_fixture
        assert isinstance(line_series, _BaseCategorySeries)

    def it_uses__MarkerMixin(self, subclass_fixture):
        line_series = subclass_fixture
        assert isinstance(line_series, _MarkerMixin)

    def it_knows_whether_it_should_use_curve_smoothing(self, smooth_get_fixture):
        series, expected_value = smooth_get_fixture
        assert series.smooth == expected_value

    def it_can_change_whether_it_uses_curve_smoothing(self, smooth_set_fixture):
        series, new_value, expected_xml = smooth_set_fixture
        series.smooth = new_value
        assert series._element.xml == expected_xml

    # fixtures -------------------------------------------------------

    @pytest.fixture(
        params=[
            ("c:ser", True),
            ("c:ser/c:smooth", True),
            ("c:ser/c:smooth{val=1}", True),
            ("c:ser/c:smooth{val=0}", False),
        ]
    )
    def smooth_get_fixture(self, request):
        ser_cxml, expected_value = request.param
        series = LineSeries(element(ser_cxml))
        return series, expected_value

    @pytest.fixture(
        params=[
            ("c:ser", True, "c:ser/c:smooth"),
            ("c:ser/c:smooth", False, "c:ser/c:smooth{val=0}"),
            ("c:ser/c:smooth{val=0}", True, "c:ser/c:smooth"),
        ]
    )
    def smooth_set_fixture(self, request):
        ser_cxml, new_value, expected_ser_cxml = request.param
        series = LineSeries(element(ser_cxml))
        expected_xml = xml(expected_ser_cxml)
        return series, new_value, expected_xml

    @pytest.fixture
    def subclass_fixture(self):
        return LineSeries(None)


class DescribePieSeries(object):
    def it_is_a_BaseCategorySeries_subclass(self, subclass_fixture):
        pie_series = subclass_fixture
        assert isinstance(pie_series, _BaseCategorySeries)

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def subclass_fixture(self):
        return PieSeries(None)


class DescribeRadarSeries(object):
    def it_is_a_BaseCategorySeries_subclass(self, subclass_fixture):
        radar_series = subclass_fixture
        assert isinstance(radar_series, _BaseCategorySeries)

    def it_uses__MarkerMixin(self, subclass_fixture):
        line_series = subclass_fixture
        assert isinstance(line_series, _MarkerMixin)

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def subclass_fixture(self):
        return RadarSeries(None)


class Describe_XySeries(object):
    def it_uses__MarkerMixin(self, subclass_fixture):
        line_series = subclass_fixture
        assert isinstance(line_series, _MarkerMixin)

    def it_provides_access_to_its_points(self, points_fixture):
        series, XyPoints_, ser, points_ = points_fixture
        points = series.points
        XyPoints_.assert_called_once_with(ser)
        assert points is points_

    def it_knows_its_values(self, values_get_fixture):
        series, expected_values = values_get_fixture
        assert series.values == expected_values

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def points_fixture(self, XyPoints_, points_):
        ser = element("c:ser")
        series = XySeries(ser)
        return series, XyPoints_, ser, points_

    @pytest.fixture
    def subclass_fixture(self):
        return XySeries(None)

    @pytest.fixture(
        params=[
            ("c:ser", ()),
            ("c:ser/c:yVal/c:numRef", ()),
            ("c:ser/c:val/c:numRef/c:numCache", ()),
            (
                "c:ser/c:yVal/c:numRef/c:numCache/(c:ptCount{val=1},c:pt{idx=0}/c:v"
                '"1.1")',
                (1.1,),
            ),
            (
                "c:ser/c:yVal/c:numRef/c:numCache/(c:ptCount{val=3},c:pt{idx=0}/c:v"
                '"1.1",c:pt{idx=2}/c:v"3.3")',
                (1.1, None, 3.3),
            ),
            ("c:ser/c:val/c:numLit", ()),
            (
                'c:ser/c:yVal/c:numLit/(c:ptCount{val=3},c:pt{idx=0}/c:v"1.1",c:pt{'
                'idx=2}/c:v"3.3")',
                (1.1, None, 3.3),
            ),
        ]
    )
    def values_get_fixture(self, request):
        ser_cxml, expected_values = request.param
        series = XySeries(element(ser_cxml))
        return series, expected_values

    # fixture components ---------------------------------------------

    @pytest.fixture
    def XyPoints_(self, request, points_):
        return class_mock(request, "pptx.chart.series.XyPoints", return_value=points_)

    @pytest.fixture
    def points_(self, request):
        return instance_mock(request, XyPoints)


class DescribeSeriesCollection(object):
    def it_supports_indexed_access(self, getitem_fixture):
        series_collection, index, _SeriesFactory_, ser, series_ = getitem_fixture
        series = series_collection[index]
        _SeriesFactory_.assert_called_once_with(ser)
        assert series is series_

    def it_supports_len(self, len_fixture):
        series_collection, expected_len = len_fixture
        assert len(series_collection) == expected_len

    # fixtures -------------------------------------------------------

    @pytest.fixture(
        params=[
            ("c:barChart/c:ser/c:order{val=42}", 0, 0),
            (
                "c:barChart/(c:ser/c:order{val=9},c:ser/c:order{val=6},c:ser/c:orde"
                "r{val=3})",
                2,
                0,
            ),
            (
                "c:plotArea/(c:barChart/(c:ser/(c:idx{val=1},c:order{val=3}),c:ser/"
                "(c:idx{val=0},c:order{val=0})),c:lineChart/c:ser/(c:idx{val=2},c:o"
                "rder{val=1}))",
                1,
                0,
            ),
        ]
    )
    def getitem_fixture(self, request, _SeriesFactory_, series_):
        cxml, index, _offset = request.param
        parent_elm = element(cxml)
        ser = parent_elm.xpath(".//c:ser")[_offset]
        series_collection = SeriesCollection(parent_elm)
        return series_collection, index, _SeriesFactory_, ser, series_

    @pytest.fixture(
        params=[
            ("c:barChart", 0),
            ("c:barChart/c:ser/c:order{val=4}", 1),
            (
                "c:barChart/(c:ser/c:order{val=4},c:ser/c:order{val=1},c:ser/c:orde"
                "r{val=6})",
                3,
            ),
            ("c:plotArea/c:barChart", 0),
            ("c:plotArea/c:barChart/c:ser/c:order{val=4}", 1),
            (
                "c:plotArea/c:barChart/(c:ser/c:order{val=4},c:ser/c:order{val=1},c"
                ":ser/c:order{val=6})",
                3,
            ),
        ]
    )
    def len_fixture(self, request):
        cxml, expected_len = request.param
        series_collection = SeriesCollection(element(cxml))
        return series_collection, expected_len

    # fixture components ---------------------------------------------

    @pytest.fixture
    def _SeriesFactory_(self, request, series_):
        return function_mock(
            request, "pptx.chart.series._SeriesFactory", return_value=series_
        )

    @pytest.fixture
    def series_(self, request):
        return instance_mock(request, _BaseSeries)


class Describe_SeriesFactory(object):
    def it_contructs_a_series_object_from_a_plot_element(self, call_fixture):
        ser, SeriesCls_, series_ = call_fixture
        series = _SeriesFactory(ser)
        SeriesCls_.assert_called_once_with(ser)
        assert series is series_

    # fixtures -------------------------------------------------------

    @pytest.fixture(
        params=[
            ("c:areaChart/c:ser", "AreaSeries"),
            ("c:barChart/c:ser", "BarSeries"),
            ("c:bubbleChart/c:ser", "BubbleSeries"),
            ("c:doughnutChart/c:ser", "PieSeries"),
            ("c:lineChart/c:ser", "LineSeries"),
            ("c:pieChart/c:ser", "PieSeries"),
            ("c:radarChart/c:ser", "RadarSeries"),
            ("c:scatterChart/c:ser", "XySeries"),
        ]
    )
    def call_fixture(self, request):
        xChart_cxml, cls_name = request.param
        ser = element(xChart_cxml).ser_lst[0]
        SeriesCls_ = class_mock(request, "pptx.chart.series.%s" % cls_name)
        series_ = SeriesCls_.return_value
        return ser, SeriesCls_, series_



# encoding: utf-8

"""
Gherkin step implementations for presentation-level features.
"""

from __future__ import absolute_import

import os

from behave import given, when, then

from pptx import Presentation
from pptx.compat import BytesIO
from pptx.opc.constants import RELATIONSHIP_TYPE as RT
from pptx.util import Inches

from helpers import saved_pptx_path, test_pptx


# given ===================================================


@given("a clean working directory")
def given_clean_working_dir(context):
    if os.path.isfile(saved_pptx_path):
        os.remove(saved_pptx_path)


@given("a presentation")
def given_a_presentation(context):
    context.presentation = Presentation(test_pptx("prs-properties"))


@given("a presentation having a notes master")
def given_a_presentation_having_a_notes_master(context):
    context.prs = Presentation(test_pptx("prs-notes"))


@given("a presentation having no notes master")
def given_a_presentation_having_no_notes_master(context):
    context.prs = Presentation(test_pptx("prs-properties"))


@given("a presentation with external relationships")
def given_prs_with_ext_rels(context):
    context.prs = Presentation(test_pptx("ext-rels"))


@given("an initialized pptx environment")
def given_initialized_pptx_env(context):
    pass


# when ====================================================


@when("I change the slide width and height")
def when_change_slide_width_and_height(context):
    presentation = context.presentation
    presentation.slide_width = Inches(4)
    presentation.slide_height = Inches(3)


@when("I construct a Presentation instance with no path argument")
def when_construct_default_prs(context):
    context.prs = Presentation()


@when("I open a basic PowerPoint presentation")
def when_open_basic_pptx(context):
    context.prs = Presentation(test_pptx("test"))


@when("I open a presentation contained in a stream")
def when_open_presentation_stream(context):
    with open(test_pptx("test"), "rb") as f:
        stream = BytesIO(f.read())
    context.prs = Presentation(stream)
    stream.close()


@when("I save and reload the presentation")
def when_save_and_reload_prs(context):
    if os.path.isfile(saved_pptx_path):
        os.remove(saved_pptx_path)
    context.prs.save(saved_pptx_path)
    context.prs = Presentation(saved_pptx_path)


@when("I save that stream to a file")
def when_save_stream_to_a_file(context):
    if os.path.isfile(saved_pptx_path):
        os.remove(saved_pptx_path)
    context.stream.seek(0)
    with open(saved_pptx_path, "wb") as f:
        f.write(context.stream.read())


@when("I save the presentation")
def when_save_presentation(context):
    if os.path.isfile(saved_pptx_path):
        os.remove(saved_pptx_path)
    context.prs.save(saved_pptx_path)


@when("I save the presentation to a stream")
def when_save_presentation_to_stream(context):
    context.stream = BytesIO()
    context.prs.save(context.stream)


# then ====================================================


@then("I receive a presentation based on the default template")
def then_receive_prs_based_on_def_tmpl(context):
    prs = context.prs
    assert prs is not None
    slide_masters = prs.slide_masters
    assert slide_masters is not None
    assert len(slide_masters) == 1
    slide_layouts = slide_masters[0].slide_layouts
    assert slide_layouts is not None
    assert len(slide_layouts) == 11


@then("its slide height matches its known value")
def then_slide_height_matches_known_value(context):
    presentation = context.presentation
    assert presentation.slide_height == 6858000


@then("its slide width matches its known value")
def then_slide_width_matches_known_value(context):
    presentation = context.presentation
    assert presentation.slide_width == 9144000


@then("I see the pptx file in the working directory")
def then_see_pptx_file_in_working_dir(context):
    assert os.path.isfile(saved_pptx_path)
    minimum = 30000
    actual = os.path.getsize(saved_pptx_path)
    assert actual > minimum


@then("len(notes_master.shapes) is {shape_count}")
def then_len_notes_master_shapes_is_shape_count(context, shape_count):
    notes_master = context.prs.notes_master
    expected = int(shape_count)
    actual = len(notes_master.shapes)
    assert actual == expected, "got %s" % actual


@then("prs.notes_master is a NotesMaster object")
def then_prs_notes_master_is_a_NotesMaster_object(context):
    prs = context.prs
    assert type(prs.notes_master).__name__ == "NotesMaster"


@then("prs.slides is a Slides object")
def then_prs_slides_is_a_Slides_object(context):
    prs = context.presentation
    assert type(prs.slides).__name__ == "Slides"


@then("prs.slide_masters is a SlideMasters object")
def then_prs_slide_masters_is_a_SlideMasters_object(context):
    prs = context.presentation
    assert type(prs.slide_masters).__name__ == "SlideMasters"


@then("the external relationships are still there")
def then_ext_rels_are_preserved(context):
    prs = context.prs
    sld = prs.slides[0]
    rel = sld.part._rels["rId2"]
    assert rel.is_external
    assert rel.reltype == RT.HYPERLINK
    assert rel.target_ref == "https://github.com/scanny/python-pptx"


@then("the slide height matches the new value")
def then_slide_height_matches_new_value(context):
    presentation = context.presentation
    assert presentation.slide_height == Inches(3)


@then("the slide width matches the new value")
def then_slide_width_matches_new_value(context):
    presentation = context.presentation
    assert presentation.slide_width == Inches(4)



# encoding: utf-8

"""
Test suite for pptx.oxml.slide module
"""

from __future__ import absolute_import, division, print_function, unicode_literals

import pytest

from pptx.oxml.slide import CT_NotesMaster, CT_NotesSlide

from ..unitutil.file import snippet_text


class DescribeCT_NotesMaster(object):
    def it_can_create_a_default_notesMaster_element(self, new_fixture):
        expected_xml = new_fixture
        notesMaster = CT_NotesMaster.new_default()
        assert notesMaster.xml == expected_xml

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def new_fixture(self):
        expected_xml = snippet_text("default-notesMaster")
        return expected_xml


class DescribeCT_NotesSlide(object):
    def it_can_create_a_new_notes_element(self, new_fixture):
        expected_xml = new_fixture
        notes = CT_NotesSlide.new()
        assert notes.xml == expected_xml

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def new_fixture(self):
        expected_xml = snippet_text("default-notes")
        return expected_xml



# encoding: utf-8

"""Unit test suite for pptx.media module."""

from __future__ import absolute_import, division, print_function, unicode_literals

import pytest

from pptx.compat import BytesIO
from pptx.media import Video

from .unitutil.file import absjoin, test_file_dir
from .unitutil.mock import initializer_mock, instance_mock, method_mock, property_mock


TEST_VIDEO_PATH = absjoin(test_file_dir, "dummy.mp4")


class DescribeVideo(object):
    def it_can_construct_from_a_path(self, from_path_fixture):
        movie_path, mime_type, blob, filename, video_ = from_path_fixture
        video = Video.from_path_or_file_like(movie_path, mime_type)
        Video.from_blob.assert_called_once_with(blob, mime_type, filename)
        assert video is video_

    def it_can_construct_from_a_stream(self, from_stream_fixture):
        movie_stream, mime_type, blob, video_ = from_stream_fixture
        video = Video.from_path_or_file_like(movie_stream, mime_type)
        Video.from_blob.assert_called_once_with(blob, mime_type, None)
        assert video is video_

    def it_can_construct_from_a_blob(self, from_blob_fixture):
        blob, mime_type, filename, Video_init_ = from_blob_fixture
        video = Video.from_blob(blob, mime_type, filename)
        Video_init_.assert_called_once_with(video, blob, mime_type, filename)
        assert isinstance(video, Video)

    def it_provides_access_to_the_video_bytestream(self, blob_fixture):
        video, expected_value = blob_fixture
        assert video.blob == expected_value

    def it_knows_its_content_type(self, content_type_fixture):
        video, expected_value = content_type_fixture
        assert video.content_type == expected_value

    def it_knows_a_filename_for_the_video(self, filename_fixture):
        video, expected_value = filename_fixture
        assert video.filename == expected_value

    def it_knows_an_extension_for_the_video(self, ext_fixture):
        video, expected_value = ext_fixture
        assert video.ext == expected_value

    def it_knows_its_sha1_hash(self, sha1_fixture):
        video, expected_value = sha1_fixture
        assert video.sha1 == expected_value

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def blob_fixture(self):
        blob = b"blob-bytes"
        video = Video(blob, None, None)
        expected_value = blob
        return video, expected_value

    @pytest.fixture
    def content_type_fixture(self):
        mime_type = "video/mp4"
        video = Video(None, mime_type, None)
        expected_value = mime_type
        return video, expected_value

    @pytest.fixture(
        params=[
            (None, "foo.bar", "bar"),
            ("video/mp4", None, "mp4"),
            ("video/xyz", None, "vid"),
        ]
    )
    def ext_fixture(self, request):
        mime_type, filename, expected_value = request.param
        video = Video(None, mime_type, filename)
        return video, expected_value

    @pytest.fixture(
        params=[("foobar.mp4", None, "foobar.mp4"), (None, "vid", "movie.vid")]
    )
    def filename_fixture(self, request, ext_prop_):
        filename, ext, expected_value = request.param
        video = Video(None, None, filename)
        ext_prop_.return_value = ext
        return video, expected_value

    @pytest.fixture
    def from_blob_fixture(self, Video_init_):
        blob, mime_type, filename = "01234", "video/mp4", "movie.mp4"
        return blob, mime_type, filename, Video_init_

    @pytest.fixture
    def from_path_fixture(self, video_, from_blob_):
        movie_path, mime_type = TEST_VIDEO_PATH, "video/mp4"
        with open(movie_path, "rb") as f:
            blob = f.read()
        filename = "dummy.mp4"
        from_blob_.return_value = video_
        return movie_path, mime_type, blob, filename, video_

    @pytest.fixture
    def from_stream_fixture(self, video_, from_blob_):
        with open(TEST_VIDEO_PATH, "rb") as f:
            blob = f.read()
            movie_stream = BytesIO(blob)
        mime_type = "video/mp4"
        from_blob_.return_value = video_
        return movie_stream, mime_type, blob, video_

    @pytest.fixture
    def sha1_fixture(self):
        blob = b"blobish"
        video = Video(blob, None, None)
        expected_value = "de731a6eed12f427642325193b8e57af3c624d62"
        return video, expected_value

    # fixture components ---------------------------------------------

    @pytest.fixture
    def ext_prop_(self, request):
        return property_mock(request, Video, "ext")

    @pytest.fixture
    def from_blob_(self, request):
        return method_mock(request, Video, "from_blob")

    @pytest.fixture
    def video_(self, request):
        return instance_mock(request, Video)

    @pytest.fixture
    def Video_init_(self, request):
        return initializer_mock(request, Video, autospec=True)



# encoding: utf-8

"""Objects related to images, audio, and video."""

from __future__ import absolute_import, division, print_function, unicode_literals

import base64
import hashlib
import os

from .compat import is_string
from .opc.constants import CONTENT_TYPE as CT
from .util import lazyproperty


class Video(object):
    """Immutable value object representing a video such as MP4."""

    def __init__(self, blob, mime_type, filename):
        super(Video, self).__init__()
        self._blob = blob
        self._mime_type = mime_type
        self._filename = filename

    @classmethod
    def from_blob(cls, blob, mime_type, filename=None):
        """Return a new |Video| object loaded from image binary in *blob*."""
        return cls(blob, mime_type, filename)

    @classmethod
    def from_path_or_file_like(cls, movie_file, mime_type):
        """Return a new |Video| object containing video in *movie_file*.

        *movie_file* can be either a path (string) or a file-like
        (e.g. StringIO) object.
        """
        if is_string(movie_file):
            # treat movie_file as a path
            with open(movie_file, "rb") as f:
                blob = f.read()
            filename = os.path.basename(movie_file)
        else:
            # assume movie_file is a file-like object
            blob = movie_file.read()
            filename = None

        return cls.from_blob(blob, mime_type, filename)

    @property
    def blob(self):
        """The bytestream of the media "file"."""
        return self._blob

    @property
    def content_type(self):
        """MIME-type of this media, e.g. `'video/mp4'`."""
        return self._mime_type

    @property
    def ext(self):
        """Return the file extension for this video, e.g. 'mp4'.

        The extension is that from the actual filename if known. Otherwise
        it is the lowercase canonical extension for the video's MIME type.
        'vid' is used if the MIME type is 'video/unknown'.
        """
        if self._filename:
            return os.path.splitext(self._filename)[1].lstrip(".")
        return {
            CT.ASF: "asf",
            CT.AVI: "avi",
            CT.MOV: "mov",
            CT.MP4: "mp4",
            CT.MPG: "mpg",
            CT.MS_VIDEO: "avi",
            CT.SWF: "swf",
            CT.WMV: "wmv",
            CT.X_MS_VIDEO: "avi",
        }.get(self._mime_type, "vid")

    @property
    def filename(self):
        """Return a filename.ext string appropriate to this video.

        The base filename from the original path is used if this image was
        loaded from the filesystem. If no filename is available, such as when
        the video object is created from an in-memory stream, the string
        'movie.{ext}' is used where 'ext' is suitable to the video format,
        such as 'mp4'.
        """
        if self._filename is not None:
            return self._filename
        return "movie.%s" % self.ext

    @lazyproperty
    def sha1(self):
        """The SHA1 hash digest for the binary "file" of this video.

        Example: `'1be010ea47803b00e140b852765cdf84f491da47'`
        """
        return hashlib.sha1(self._blob).hexdigest()


SPEAKER_IMAGE_BYTES = base64.b64decode(
    "iVBORw0KGgoAAAANSUhEUgAAAHgAAAA3CAYAAADHao5rAAAACXBIWXMAAAsTAAALEwEAmpw"
    "YAAAKT2lDQ1BQaG90b3Nob3AgSUNDIHByb2ZpbGUAAHjanVNnVFPpFj333vRCS4iAlEtvUh"
    "UIIFJCi4AUkSYqIQkQSoghodkVUcERRUUEG8igiAOOjoCMFVEsDIoK2AfkIaKOg6OIisr74"
    "Xuja9a89+bN/rXXPues852zzwfACAyWSDNRNYAMqUIeEeCDx8TG4eQuQIEKJHAAEAizZCFz"
    "/SMBAPh+PDwrIsAHvgABeNMLCADATZvAMByH/w/qQplcAYCEAcB0kThLCIAUAEB6jkKmAEB"
    "GAYCdmCZTAKAEAGDLY2LjAFAtAGAnf+bTAICd+Jl7AQBblCEVAaCRACATZYhEAGg7AKzPVo"
    "pFAFgwABRmS8Q5ANgtADBJV2ZIALC3AMDOEAuyAAgMADBRiIUpAAR7AGDIIyN4AISZABRG8"
    "lc88SuuEOcqAAB4mbI8uSQ5RYFbCC1xB1dXLh4ozkkXKxQ2YQJhmkAuwnmZGTKBNA/g88wA"
    "AKCRFRHgg/P9eM4Ors7ONo62Dl8t6r8G/yJiYuP+5c+rcEAAAOF0ftH+LC+zGoA7BoBt/qI"
    "l7gRoXgugdfeLZrIPQLUAoOnaV/Nw+H48PEWhkLnZ2eXk5NhKxEJbYcpXff5nwl/AV/1s+X"
    "48/Pf14L7iJIEyXYFHBPjgwsz0TKUcz5IJhGLc5o9H/LcL//wd0yLESWK5WCoU41EScY5Em"
    "ozzMqUiiUKSKcUl0v9k4t8s+wM+3zUAsGo+AXuRLahdYwP2SycQWHTA4vcAAPK7b8HUKAgD"
    "gGiD4c93/+8//UegJQCAZkmScQAAXkQkLlTKsz/HCAAARKCBKrBBG/TBGCzABhzBBdzBC/x"
    "gNoRCJMTCQhBCCmSAHHJgKayCQiiGzbAdKmAv1EAdNMBRaIaTcA4uwlW4Dj1wD/phCJ7BKL"
    "yBCQRByAgTYSHaiAFiilgjjggXmYX4IcFIBBKLJCDJiBRRIkuRNUgxUopUIFVIHfI9cgI5h"
    "1xGupE7yAAygvyGvEcxlIGyUT3UDLVDuag3GoRGogvQZHQxmo8WoJvQcrQaPYw2oefQq2gP"
    "2o8+Q8cwwOgYBzPEbDAuxsNCsTgsCZNjy7EirAyrxhqwVqwDu4n1Y8+xdwQSgUXACTYEd0I"
    "gYR5BSFhMWE7YSKggHCQ0EdoJNwkDhFHCJyKTqEu0JroR+cQYYjIxh1hILCPWEo8TLxB7iE"
    "PENyQSiUMyJ7mQAkmxpFTSEtJG0m5SI+ksqZs0SBojk8naZGuyBzmULCAryIXkneTD5DPkG"
    "+Qh8lsKnWJAcaT4U+IoUspqShnlEOU05QZlmDJBVaOaUt2ooVQRNY9aQq2htlKvUYeoEzR1"
    "mjnNgxZJS6WtopXTGmgXaPdpr+h0uhHdlR5Ol9BX0svpR+iX6AP0dwwNhhWDx4hnKBmbGAc"
    "YZxl3GK+YTKYZ04sZx1QwNzHrmOeZD5lvVVgqtip8FZHKCpVKlSaVGyovVKmqpqreqgtV81"
    "XLVI+pXlN9rkZVM1PjqQnUlqtVqp1Q61MbU2epO6iHqmeob1Q/pH5Z/YkGWcNMw09DpFGgs"
    "V/jvMYgC2MZs3gsIWsNq4Z1gTXEJrHN2Xx2KruY/R27iz2qqaE5QzNKM1ezUvOUZj8H45hx"
    "+Jx0TgnnKKeX836K3hTvKeIpG6Y0TLkxZVxrqpaXllirSKtRq0frvTau7aedpr1Fu1n7gQ5"
    "Bx0onXCdHZ4/OBZ3nU9lT3acKpxZNPTr1ri6qa6UbobtEd79up+6Ynr5egJ5Mb6feeb3n+h"
    "x9L/1U/W36p/VHDFgGswwkBtsMzhg8xTVxbzwdL8fb8VFDXcNAQ6VhlWGX4YSRudE8o9VGj"
    "UYPjGnGXOMk423GbcajJgYmISZLTepN7ppSTbmmKaY7TDtMx83MzaLN1pk1mz0x1zLnm+eb"
    "15vft2BaeFostqi2uGVJsuRaplnutrxuhVo5WaVYVVpds0atna0l1rutu6cRp7lOk06rntZ"
    "nw7Dxtsm2qbcZsOXYBtuutm22fWFnYhdnt8Wuw+6TvZN9un2N/T0HDYfZDqsdWh1+c7RyFD"
    "pWOt6azpzuP33F9JbpL2dYzxDP2DPjthPLKcRpnVOb00dnF2e5c4PziIuJS4LLLpc+Lpsbx"
    "t3IveRKdPVxXeF60vWdm7Obwu2o26/uNu5p7ofcn8w0nymeWTNz0MPIQ+BR5dE/C5+VMGvf"
    "rH5PQ0+BZ7XnIy9jL5FXrdewt6V3qvdh7xc+9j5yn+M+4zw33jLeWV/MN8C3yLfLT8Nvnl+"
    "F30N/I/9k/3r/0QCngCUBZwOJgUGBWwL7+Hp8Ib+OPzrbZfay2e1BjKC5QRVBj4KtguXBrS"
    "FoyOyQrSH355jOkc5pDoVQfujW0Adh5mGLw34MJ4WHhVeGP45wiFga0TGXNXfR3ENz30T6R"
    "JZE3ptnMU85ry1KNSo+qi5qPNo3ujS6P8YuZlnM1VidWElsSxw5LiquNm5svt/87fOH4p3i"
    "C+N7F5gvyF1weaHOwvSFpxapLhIsOpZATIhOOJTwQRAqqBaMJfITdyWOCnnCHcJnIi/RNtG"
    "I2ENcKh5O8kgqTXqS7JG8NXkkxTOlLOW5hCepkLxMDUzdmzqeFpp2IG0yPTq9MYOSkZBxQq"
    "ohTZO2Z+pn5mZ2y6xlhbL+xW6Lty8elQfJa7OQrAVZLQq2QqboVFoo1yoHsmdlV2a/zYnKO"
    "ZarnivN7cyzytuQN5zvn//tEsIS4ZK2pYZLVy0dWOa9rGo5sjxxedsK4xUFK4ZWBqw8uIq2"
    "Km3VT6vtV5eufr0mek1rgV7ByoLBtQFr6wtVCuWFfevc1+1dT1gvWd+1YfqGnRs+FYmKrhT"
    "bF5cVf9go3HjlG4dvyr+Z3JS0qavEuWTPZtJm6ebeLZ5bDpaql+aXDm4N2dq0Dd9WtO319k"
    "XbL5fNKNu7g7ZDuaO/PLi8ZafJzs07P1SkVPRU+lQ27tLdtWHX+G7R7ht7vPY07NXbW7z3/"
    "T7JvttVAVVN1WbVZftJ+7P3P66Jqun4lvttXa1ObXHtxwPSA/0HIw6217nU1R3SPVRSj9Yr"
    "60cOxx++/p3vdy0NNg1VjZzG4iNwRHnk6fcJ3/ceDTradox7rOEH0x92HWcdL2pCmvKaRpt"
    "TmvtbYlu6T8w+0dbq3nr8R9sfD5w0PFl5SvNUyWna6YLTk2fyz4ydlZ19fi753GDborZ752"
    "PO32oPb++6EHTh0kX/i+c7vDvOXPK4dPKy2+UTV7hXmq86X23qdOo8/pPTT8e7nLuarrlca"
    "7nuer21e2b36RueN87d9L158Rb/1tWeOT3dvfN6b/fF9/XfFt1+cif9zsu72Xcn7q28T7xf"
    "9EDtQdlD3YfVP1v+3Njv3H9qwHeg89HcR/cGhYPP/pH1jw9DBY+Zj8uGDYbrnjg+OTniP3L"
    "96fynQ89kzyaeF/6i/suuFxYvfvjV69fO0ZjRoZfyl5O/bXyl/erA6xmv28bCxh6+yXgzMV"
    "70VvvtwXfcdx3vo98PT+R8IH8o/2j5sfVT0Kf7kxmTk/8EA5jz/GMzLdsAAAAgY0hSTQAAe"
    "iUAAICDAAD5/wAAgOkAAHUwAADqYAAAOpgAABdvkl/FRgAACJ5JREFUeNrsm19oW9cdx7/n"
    "/tO9uleS9ceWZMdO68Su0jiEmNCGLVsDg+J16xhkYx0UM1ayvS19SQZ7WfK6PWR7Gax0S9n"
    "24JGFlbGNPIRgEkKWZC1u6rizHMXxv1iWJdnWvbqS7p9z9uDokqzs1bWd84GDQEigez465/"
    "x+v3MOYYyBs3shXDAXzOGCOVwwhwvmcMEcLpjDBXPBHC6YwwVzuOAt4dGjRy/MzMwMmqZpS"
    "JLkZbPZYn9//8NkMlnmgncwt27d+tKlS5e+W6lUEolEYjQUCoExBsuy4Lru+319fXMjIyNX"
    "jh49+m8ueAfRarXUCxcuvHvv3r3DR44ceavRaMCyLDDGIAgCFEVBOBwGIQSFQuH9PXv2PD5"
    "16tRvu7u7H3PB2xzXdZXz58//vFKp/Gzfvn1YXFyEYRgwDAOSJEEQBDDG4LouPM+DpmnwPA"
    "/5fP73o6Ojf3zttdfGueBtzAcffPCD8fHxi0NDQ1hZWUE6nYYoiiCEQBCEZ14JIfA8D77vA"
    "wAmJib+cPLkyctvvvnm33ZLf0i7Se7s7Gz/tWvXvpbL5bC+vo5sNgtFUTb/yU/EthshBAAg"
    "yzIAoNls4tChQ6OXL1+GKIreG2+88U8ueJsxPj5+QlXVt0OhEGRZhqIoEEXxGbFPC6aUBuJ"
    "VVYVlWThw4MDo2NiYkMlkisPDwx/v9D4Rdotc0zSj9+7dO5RMJgEA4XAYkiRBkqRA9tNNlu"
    "VArKIokCQJ8Xgc8Xgc+/fvf/u999778fr6egcXvE0ol8spx3HeNQwDoihClmXIsgxRFCFJ0"
    "ucEE0KgqipCodAz0pPJJFKpFGKx2I8uXrz4Qy54m1CpVBK+70MUxUDW0yO1Lbz9XnuUPz26"
    "25/JZDLo7OzExMTE4cnJySG+Bm8DarVa1Pd9EEIgy3IwPYuiCFEANFWEIGw2x6PQNA2qqqK"
    "dRTDGwBgDpTSQXKvVRj/88MOZoaGhSS74C8a27XA75ZEk6YloBYoswHQNfPy4Fx4JoTdmYj"
    "BRhSozsP+ZwCil8DwPlFIkk0kkEglMTEwM5PP5wcHBwTwX/AXhOI5i23bY930wxqAoymbVi"
    "jA0/DD+mj8C048iqjMUHYbFRg0n+uaQ0FzQJ5IdxwGlNFi/AaCzsxPpdHr0+vXrN3aq4F2x"
    "Bs/Pz/eFQqE/t0egIAjQdR2SQDFVzmKlHoEme1BEH3uyDJFsHFdnX4SLECRRhOM4EAQBhmF"
    "A1/UgvUomk4jH4/j0008PeZ4nccFbTKPRCH/00UdHi8Viph1gNZtNUEqhKAp0XUfRNMAYQC"
    "ng+0DNAhp1H4s1A3fmkwhrMmKxGGKxGFRVDdZvWZYRiUQQjUZRq9V+Mjc39wIXvIWsra0lx"
    "sfHTzx48OAuY+yGKIpQVRX1eh2O4wQBFmMUjsvQ8oCWC5RWGWZmKVyX4e58DLarIBzWnomy"
    "23mxoiiIRqPQdR2FQqGfr8FbhOu6SrFYzJim+Y9mswnHcSCKImKxGFZWVmBZFgRBgCQTpPU"
    "6LNOCquhgjMERAUIAnzJUWgrmqzJeTclouRSUUrQj8XbQZRgGwuEwFhYW+vgI3iJarZbS2d"
    "l5v1Qqwfd9uK4LAEgmk2g0GlhfX99cV0UFL2dNCLSGmtlEvUFg2oBpA5YNmHWCikmgKNLnc"
    "uH2VN2udFWr1QQXvEV4nicxxlCv19FsNoOtv0QiAUVRsLq6ikqlAtel2JvycOxFCyulFZh1"
    "CqtBNuXaBPV6EyHRhyAIaLVaQST99MZEWzQPsrZ2BKu2bYMQgmq1Cs/z4DgOwuEwMpkMSqU"
    "SlpeXwQBoegTfe9XEgayDhaUHWNuoo1YHKus1hEgVuR4GLRxFPB6HrusQRRGU0uBwQLsAsl"
    "O3VXdskNVsNtHR0YHl5WU0Gg24rgvGGLLZLCilWFpawuLiIggR0d1l4KffMPH6yz5a5iNUV"
    "qcR8ot4+9UKcv0pCOLmlBwKhaBpGjRNC1Kl9p9H07QmD7K2CFVVm2tra0in07AsCysrK0il"
    "UnBdF5FIBD09PVhYWMDDhw+RyWTQ07MH/X0qznyripNFhnKNYU8SeOVwL6Idm9+jdFNm+yB"
    "AO9CybRumaeLw4cOPueAtIhKJ1Hzf/4phGDe6urowOTmJY8eOBbXk9jRdKBQQiUTQ0dGBeD"
    "yOSCyB4z0aZEmErKhoNDfX7v83BXueh2q1ilKpBF7J2sofLQi0p6dn0bZtDAwMYGNjA1NTU"
    "/B9H61WC4qioK+vDxsbG5iZmcHdu3dh2zZEUUTL8QFBAYgIQSAolUrBsR1KadCe5NpYXl6G"
    "qqp/yuVy/+GCt5C9e/fOy7L8dVmWkcvlMD09jc8++yxYM3VdR29vL+bn53H//n3cunULtm0"
    "HGwqu6wY169XVVTiOE4hmjKHRaGB2dhaFQgHHjx+/EQ6H7Z3YT+K5c+d2pGBCCOvu7n48NT"
    "X1gFL6bcMwUCgUUCqVgkpUu/Q4NzeHZrOJVquFSCQCwzCCIzuSJMH3fViWBUII2ulXoVDAn"
    "Tt3UC6X/3L27NlfaJrW4IK3GFmW3ZdeeilvWda/HMe5u2/fvhFFUWCa5m/279///vDw8K8y"
    "mczfCSH56enpr5bL5UC0IAhPSpksCKparRbK5TJmZmZw+/ZtfPLJJ1fPnDnzy506PQO76Nh"
    "stVpNFIvFDGNMSKfTxVQqVX66tHnp0qXvjI2NfZ8Q8s3e3l709/ejq6sL0WgUoVAIruuiVq"
    "thaWkJ09PTWFpaunL69Olfj4yMXNnJ/fJcXT7L5/ODY2Njb928efPLoii+3t5IkGUZnufBN"
    "E2sra1dHR4e/vidd9753cDAQH6nP/Nzebtwdna2//bt269MTk4eKpVKXZ7nSbquW7lcbvrE"
    "iRPjBw8enNwtz/rcXx9ljAlPypJ0Nz4fvx+8y+GCuWAOF8zhgjlcMIcL5nDBHC6YC+ZwwRw"
    "umLMN+O8AX65uqCMleo4AAAAASUVORK5CYII="
)



# encoding: utf-8

"""
Core properties part, corresponds to ``/docProps/core.xml`` part in package.
"""

from __future__ import absolute_import, division, print_function, unicode_literals

from datetime import datetime

from ..opc.constants import CONTENT_TYPE as CT
from ..opc.package import XmlPart
from ..opc.packuri import PackURI
from ..oxml.coreprops import CT_CoreProperties


class CorePropertiesPart(XmlPart):
    """
    Corresponds to part named ``/docProps/core.xml``, containing the core
    document properties for this document package.
    """

    @classmethod
    def default(cls):
        core_props = cls._new()
        core_props.title = "PowerPoint Presentation"
        core_props.last_modified_by = "python-pptx"
        core_props.revision = 1
        core_props.modified = datetime.utcnow()
        return core_props

    @property
    def author(self):
        return self._element.author_text

    @author.setter
    def author(self, value):
        self._element.author_text = value

    @property
    def category(self):
        return self._element.category_text

    @category.setter
    def category(self, value):
        self._element.category_text = value

    @property
    def comments(self):
        return self._element.comments_text

    @comments.setter
    def comments(self, value):
        self._element.comments_text = value

    @property
    def content_status(self):
        return self._element.contentStatus_text

    @content_status.setter
    def content_status(self, value):
        self._element.contentStatus_text = value

    @property
    def created(self):
        return self._element.created_datetime

    @created.setter
    def created(self, value):
        self._element.created_datetime = value

    @property
    def identifier(self):
        return self._element.identifier_text

    @identifier.setter
    def identifier(self, value):
        self._element.identifier_text = value

    @property
    def keywords(self):
        return self._element.keywords_text

    @keywords.setter
    def keywords(self, value):
        self._element.keywords_text = value

    @property
    def language(self):
        return self._element.language_text

    @language.setter
    def language(self, value):
        self._element.language_text = value

    @property
    def last_modified_by(self):
        return self._element.lastModifiedBy_text

    @last_modified_by.setter
    def last_modified_by(self, value):
        self._element.lastModifiedBy_text = value

    @property
    def last_printed(self):
        return self._element.lastPrinted_datetime

    @last_printed.setter
    def last_printed(self, value):
        self._element.lastPrinted_datetime = value

    @property
    def modified(self):
        return self._element.modified_datetime

    @modified.setter
    def modified(self, value):
        self._element.modified_datetime = value

    @property
    def revision(self):
        return self._element.revision_number

    @revision.setter
    def revision(self, value):
        self._element.revision_number = value

    @property
    def subject(self):
        return self._element.subject_text

    @subject.setter
    def subject(self, value):
        self._element.subject_text = value

    @property
    def title(self):
        return self._element.title_text

    @title.setter
    def title(self, value):
        self._element.title_text = value

    @property
    def version(self):
        return self._element.version_text

    @version.setter
    def version(self, value):
        self._element.version_text = value

    @classmethod
    def _new(cls):
        partname = PackURI("/docProps/core.xml")
        content_type = CT.OPC_CORE_PROPERTIES
        core_props_elm = CT_CoreProperties.new_coreProperties()
        return CorePropertiesPart(partname, content_type, core_props_elm)



# encoding: utf-8

"""Gherkin step implementations for chart category features."""

from __future__ import absolute_import, division, print_function, unicode_literals

from behave import given, then

from pptx import Presentation

from helpers import test_pptx


# given ===================================================


@given("a Categories object containing 3 categories")
def given_a_Categories_object_containing_3_categories(context):
    prs = Presentation(test_pptx("cht-category-access"))
    context.categories = prs.slides[0].shapes[0].chart.plots[0].categories


@given("a Categories object having {count} category levels")
def given_a_Categories_object_having_count_category_levels(context, count):
    slide_idx = [2, 0, 3, 1][int(count)]
    slide = Presentation(test_pptx("cht-category-access")).slides[slide_idx]
    context.categories = slide.shapes[0].chart.plots[0].categories


@given("a Categories object having {leafs} categories and {levels} levels")
def given_a_Categories_obj_having_leafs_and_levels(context, leafs, levels):
    slide_idx = {(3, 1): 0, (8, 3): 1, (0, 0): 2, (4, 2): 3}[(int(leafs), int(levels))]
    slide = Presentation(test_pptx("cht-category-access")).slides[slide_idx]
    context.categories = slide.shapes[0].chart.plots[0].categories


@given("a Category object having idx value {idx}")
def given_a_Category_object_having_idx_value_idx(context, idx):
    cat_offset = int(idx)
    slide = Presentation(test_pptx("cht-category-access")).slides[0]
    context.category = slide.shapes[0].chart.plots[0].categories[cat_offset]


@given("a Category object having {label}")
def given_a_Category_object_having_label(context, label):
    cat_offset = {"label 'Foo'": 0, "no label": 1}[label]
    slide = Presentation(test_pptx("cht-category-access")).slides[0]
    context.category = slide.shapes[0].chart.plots[0].categories[cat_offset]


@given("a CategoryLevel object containing 4 categories")
def given_a_CategoryLevel_object_containing_4_categories(context):
    slide = Presentation(test_pptx("cht-category-access")).slides[1]
    chart = slide.shapes[0].chart
    context.category_level = chart.plots[0].categories.levels[1]


# then ====================================================


@then("categories[2] is a Category object")
def then_categories_2_is_a_Category_object(context):
    type_name = type(context.categories[2]).__name__
    assert type_name == "Category", "got %s" % type_name


@then("categories.depth is {value}")
def then_categories_depth_is_value(context, value):
    expected_value = int(value)
    depth = context.categories.depth
    assert depth == expected_value, "got %s" % expected_value


@then("categories.flattened_labels is a tuple of {leafs} tuples")
def then_categories_flattened_labels_is_tuple_of_tuples(context, leafs):
    flattened_labels = context.categories.flattened_labels
    type_name = type(flattened_labels).__name__
    length = len(flattened_labels)
    assert type_name == "tuple", "got %s" % type_name
    assert length == int(leafs), "got %s" % length
    for labels in flattened_labels:
        type_name = type(labels).__name__
        assert type_name == "tuple", "got %s" % type_name


@then("categories.levels contains {count} CategoryLevel objects")
def then_categories_levels_contains_count_CategoryLevel_objs(context, count):
    expected_idx = int(count) - 1
    idx = -1
    for idx, category_level in enumerate(context.categories.levels):
        type_name = type(category_level).__name__
        assert type_name == "CategoryLevel", "got %s" % type_name
    assert idx == expected_idx, "got %s" % idx


@then("category.idx is {value}")
def then_category_idx_is_value(context, value):
    expected_value = None if value == "None" else int(value)
    idx = context.category.idx
    assert idx == expected_value, "got %s" % idx


@then("category.label is {value}")
def then_category_label_is_value(context, value):
    expected_value = {"'Foo'": "Foo", "''": ""}[value]
    label = context.category.label
    assert label == expected_value, "got %s" % label


@then("category_level[2] is a Category object")
def then_category_level_2_is_a_Category_object(context):
    type_name = type(context.category_level[2]).__name__
    assert type_name == "Category", "got %s" % type_name


@then("each label tuple contains {levels} labels")
def then_each_label_tuple_contains_levels_labels(context, levels):
    flattened_labels = context.categories.flattened_labels
    for labels in flattened_labels:
        length = len(labels)
        assert length == int(levels), "got %s" % levels
        for label in labels:
            type_name = type(label).__name__
            assert type_name == "str", "got %s" % type_name


@then("iterating categories produces 3 Category objects")
def then_iterating_categories_produces_3_category_objects(context):
    categories = context.categories
    idx = -1
    for idx, category in enumerate(categories):
        assert type(category).__name__ == "Category"
    assert idx == 2, "got %s" % idx


@then("iterating category_level produces 4 Category objects")
def then_iterating_category_level_produces_4_Category_objects(context):
    idx = -1
    for idx, category in enumerate(context.category_level):
        type_name = type(category).__name__
        assert type_name == "Category", "got %s" % type_name
    assert idx == 3, "got %s" % idx


@then("len(categories) is {count}")
def then_len_categories_is_count(context, count):
    expected_count = int(count)
    assert len(context.categories) == expected_count


@then("len(category_level) is {count}")
def then_len_category_level_is_count(context, count):
    expected_count = int(count)
    actual_count = len(context.category_level)
    assert actual_count == expected_count, "got %s" % actual_count


@then("list(categories) == ['Foo', '', 'Baz']")
def then_list_categories_is_Foo_empty_Baz(context):
    cats_list = list(context.categories)
    assert cats_list == ["Foo", "", "Baz"], "got %s" % cats_list



# encoding: utf-8

"""
XML test data builders for pptx.oxml.text unit tests
"""

from __future__ import absolute_import, print_function

from ...unitdata import BaseBuilder


class CT_Hyperlink(BaseBuilder):
    __tag__ = "a:hlinkClick"
    __nspfxs__ = ("a", "r")
    __attrs__ = (
        "r:id",
        "invalidUrl",
        "action",
        "tgtFrame",
        "tooltip",
        "history",
        "highlightClick",
        "endSnd",
    )

    def with_rId(self, rId):
        self._set_xmlattr("r:id", rId)
        return self


class CT_OfficeArtExtensionList(BaseBuilder):
    __tag__ = "a:extLst"
    __nspfxs__ = ("a",)
    __attrs__ = ()


class CT_RegularTextRunBuilder(BaseBuilder):
    __tag__ = "a:r"
    __nspfxs__ = ("a",)
    __attrs__ = ()


class CT_TextBodyBuilder(BaseBuilder):
    __tag__ = "p:txBody"
    __nspfxs__ = ("p", "a")
    __attrs__ = ()


class CT_TextBodyPropertiesBuilder(BaseBuilder):
    __tag__ = "a:bodyPr"
    __nspfxs__ = ("a",)
    __attrs__ = (
        "rot",
        "spcFirstLastPara",
        "vertOverflow",
        "horzOverflow",
        "vert",
        "wrap",
        "lIns",
        "tIns",
        "rIns",
        "bIns",
        "numCol",
        "spcCol",
        "rtlCol",
        "fromWordArt",
        "anchor",
        "anchorCtr",
        "forceAA",
        "upright",
        "compatLnSpc",
    )


class CT_TextCharacterPropertiesBuilder(BaseBuilder):
    """
    Test data builder for CT_TextCharacterProperties XML element that appears
    as <a:endParaRPr> child of <a:p> and <a:rPr> child of <a:r>.
    """

    __nspfxs__ = ("a",)
    __attrs__ = ("b", "i", "sz", "u")

    def __init__(self, tag):
        self.__tag__ = tag
        super(CT_TextCharacterPropertiesBuilder, self).__init__()


class CT_TextFontBuilder(BaseBuilder):
    __tag__ = "a:latin"
    __nspfxs__ = ("a",)
    __attrs__ = ("typeface", "panose", "pitchFamily", "charset")


class CT_TextNoAutofitBuilder(BaseBuilder):
    __tag__ = "a:noAutofit"
    __nspfxs__ = ("a",)
    __attrs__ = ()


class CT_TextNormalAutofitBuilder(BaseBuilder):
    __tag__ = "a:normAutofit"
    __nspfxs__ = ("a",)
    __attrs__ = ("fontScale", "lnSpcReduction")


class CT_TextParagraphBuilder(BaseBuilder):
    """
    Test data builder for CT_TextParagraph (<a:p>) XML element that appears
    as a child of <p:txBody>.
    """

    __tag__ = "a:p"
    __nspfxs__ = ("a",)
    __attrs__ = ()


class CT_TextParagraphPropertiesBuilder(BaseBuilder):
    """
    Test data builder for CT_TextParagraphProperties (<a:pPr>) XML element
    that appears as a child of <a:p>.
    """

    __tag__ = "a:pPr"
    __nspfxs__ = ("a",)
    __attrs__ = (
        "marL",
        "marR",
        "lvl",
        "indent",
        "algn",
        "defTabSz",
        "rtl",
        "eaLnBrk",
        "fontAlgn",
        "latinLnBrk",
        "hangingPunct",
    )


class CT_TextShapeAutofitBuilder(BaseBuilder):
    __tag__ = "a:spAutoFit"
    __nspfxs__ = ("a",)
    __attrs__ = ()


class XsdString(BaseBuilder):
    __attrs__ = ()

    def __init__(self, tag, nspfxs):
        self.__tag__ = tag
        self.__nspfxs__ = nspfxs
        super(XsdString, self).__init__()


def a_bodyPr():
    return CT_TextBodyPropertiesBuilder()


def a_defRPr():
    return CT_TextCharacterPropertiesBuilder("a:defRPr")


def a_latin():
    return CT_TextFontBuilder()


def a_noAutofit():
    return CT_TextNoAutofitBuilder()


def a_normAutofit():
    return CT_TextNormalAutofitBuilder()


def a_p():
    """Return a CT_TextParagraphBuilder instance"""
    return CT_TextParagraphBuilder()


def a_pPr():
    """Return a CT_TextParagraphPropertiesBuilder instance"""
    return CT_TextParagraphPropertiesBuilder()


def a_t():
    return XsdString("a:t", ("a",))


def a_txBody():
    return CT_TextBodyBuilder()


def an_endParaRPr():
    return CT_TextCharacterPropertiesBuilder("a:endParaRPr")


def an_extLst():
    return CT_OfficeArtExtensionList()


def an_hlinkClick():
    return CT_Hyperlink()


def an_r():
    return CT_RegularTextRunBuilder()


def an_rPr():
    return CT_TextCharacterPropertiesBuilder("a:rPr")


def an_spAutoFit():
    return CT_TextShapeAutofitBuilder()



# encoding: utf-8

"""MediaPart and related objects."""

from __future__ import absolute_import, division, print_function, unicode_literals

import hashlib

from ..opc.package import Part
from ..util import lazyproperty


class MediaPart(Part):
    """A media part, containing an audio or video resource.

    A media part generally has a partname matching the regex
    ``ppt/media/media[1-9][0-9]*.*``.
    """

    @classmethod
    def new(cls, package, media):
        """Return new |MediaPart| instance containing *media*.

        *media* must be a |Media| object.
        """
        partname = package.next_media_partname(media.ext)
        return cls(partname, media.content_type, media.blob, package)

    @lazyproperty
    def sha1(self):
        """The SHA1 hash digest for the media binary of this media part.

        Example: `'1be010ea47803b00e140b852765cdf84f491da47'`
        """
        return hashlib.sha1(self._blob).hexdigest()



# encoding: utf-8

"""
Test data for autoshape-related unit tests.
"""

from __future__ import absolute_import

from ...unitdata import BaseBuilder


class CT_ApplicationNonVisualDrawingPropsBuilder(BaseBuilder):
    __tag__ = "p:nvPr"
    __nspfxs__ = ("p",)
    __attrs__ = ("isPhoto", "userDrawn")


class CT_ConnectorBuilder(BaseBuilder):
    __tag__ = "p:cxnSp"
    __nspfxs__ = ("p", "a")
    __attrs__ = ()


class CT_GeomGuideBuilder(BaseBuilder):
    __tag__ = "a:gd"
    __nspfxs__ = ("a",)
    __attrs__ = ("name", "fmla")


class CT_GeomGuideListBuilder(BaseBuilder):
    __tag__ = "a:avLst"
    __nspfxs__ = ("a",)
    __attrs__ = ()


class CT_GraphicalObjectBuilder(BaseBuilder):
    __tag__ = "a:graphic"
    __nspfxs__ = ("a",)
    __attrs__ = ()


class CT_GraphicalObjectDataBuilder(BaseBuilder):
    __tag__ = "a:graphicData"
    __nspfxs__ = ("a",)
    __attrs__ = ("uri",)


class CT_GraphicalObjectFrameBuilder(BaseBuilder):
    __tag__ = "p:graphicFrame"
    __nspfxs__ = ("p", "a")
    __attrs__ = ("bwMode",)


class CT_GraphicalObjectFrameNonVisualBuilder(BaseBuilder):
    __tag__ = "p:nvGraphicFramePr"
    __nspfxs__ = ("p",)
    __attrs__ = ()


class CT_GroupShapeBuilder(BaseBuilder):
    __nspfxs__ = ("p",)
    __attrs__ = ()

    def __init__(self, tag):
        self.__tag__ = tag
        super(CT_GroupShapeBuilder, self).__init__()


class CT_GroupShapePropertiesBuilder(BaseBuilder):
    __tag__ = "p:grpSpPr"
    __nspfxs__ = ("p", "a")
    __attrs__ = ("bwMode",)


class CT_NonVisualDrawingPropsBuilder(BaseBuilder):
    __tag__ = "p:cNvPr"
    __nspfxs__ = ("p",)
    __attrs__ = ("id", "name", "descr", "hidden", "title")


class CT_NonVisualDrawingShapePropsBuilder(BaseBuilder):
    __tag__ = "p:cNvSpPr"
    __nspfxs__ = ("p",)
    __attrs__ = ("txBox",)


class CT_PictureBuilder(BaseBuilder):
    __tag__ = "p:pic"
    __nspfxs__ = ("p", "a")
    __attrs__ = ()


class CT_PictureNonVisualBuilder(BaseBuilder):
    __tag__ = "p:nvPicPr"
    __nspfxs__ = ("p",)
    __attrs__ = ()


class CT_PlaceholderBuilder(BaseBuilder):
    __tag__ = "p:ph"
    __nspfxs__ = ("p",)
    __attrs__ = ("type", "orient", "sz", "idx", "hasCustomPropt")


class CT_Point2DBuilder(BaseBuilder):
    __tag__ = "a:off"
    __nspfxs__ = ("a",)
    __attrs__ = ("x", "y")


class CT_PositiveSize2DBuilder(BaseBuilder):
    __tag__ = "a:ext"
    __nspfxs__ = ("a",)
    __attrs__ = ("cx", "cy")


class CT_PresetGeometry2DBuilder(BaseBuilder):
    __tag__ = "a:prstGeom"
    __nspfxs__ = ("a",)
    __attrs__ = ("prst",)


class CT_ShapeBuilder(BaseBuilder):
    __tag__ = "p:sp"
    __nspfxs__ = ("p", "a")
    __attrs__ = ("useBgFill",)


class CT_ShapeNonVisualBuilder(BaseBuilder):
    __tag__ = "p:nvSpPr"
    __nspfxs__ = ("p",)
    __attrs__ = ()


class CT_ShapePropertiesBuilder(BaseBuilder):
    __tag__ = "p:spPr"
    __nspfxs__ = ("p", "a")
    __attrs__ = ("bwMode",)


class CT_Transform2DBuilder(BaseBuilder):
    __nspfxs__ = ("a",)
    __attrs__ = ("rot", "flipH", "flipV")

    def __init__(self, tag):
        self.__tag__ = tag
        super(CT_Transform2DBuilder, self).__init__()


def a_cNvPr():
    return CT_NonVisualDrawingPropsBuilder()


def a_cNvSpPr():
    return CT_NonVisualDrawingShapePropsBuilder()


def a_cxnSp():
    return CT_ConnectorBuilder()


def a_gd():
    return CT_GeomGuideBuilder()


def a_graphic():
    return CT_GraphicalObjectBuilder()


def a_graphicData():
    return CT_GraphicalObjectDataBuilder()


def a_graphicFrame():
    return CT_GraphicalObjectFrameBuilder()


def a_grpSp():
    return CT_GroupShapeBuilder("p:grpSp")


def a_grpSpPr():
    return CT_GroupShapePropertiesBuilder()


def a_p_xfrm():
    return CT_Transform2DBuilder("p:xfrm")


def a_ph():
    return CT_PlaceholderBuilder()


def a_pic():
    return CT_PictureBuilder()


def a_prstGeom():
    return CT_PresetGeometry2DBuilder()


def an_avLst():
    return CT_GeomGuideListBuilder()


def an_ext():
    return CT_PositiveSize2DBuilder()


def an_nvGraphicFramePr():
    return CT_GraphicalObjectFrameNonVisualBuilder()


def an_nvPicPr():
    return CT_PictureNonVisualBuilder()


def an_nvPr():
    return CT_ApplicationNonVisualDrawingPropsBuilder()


def an_nvSpPr():
    return CT_ShapeNonVisualBuilder()


def an_off():
    return CT_Point2DBuilder()


def an_sp():
    return CT_ShapeBuilder()


def an_spPr():
    return CT_ShapePropertiesBuilder()


def an_xfrm():
    return CT_Transform2DBuilder("a:xfrm")



# encoding: utf-8

"""Constant values related to the Open Packaging Convention.

In particular, this includes content (MIME) types and relationship types.
"""


class CONTENT_TYPE(object):
    """Content type URIs (like MIME-types) that specify a part's format."""

    ASF = "video/x-ms-asf"
    AVI = "video/avi"
    BMP = "image/bmp"
    DML_CHART = "application/vnd.openxmlformats-officedocument.drawingml.chart+xml"
    DML_CHARTSHAPES = (
        "application/vnd.openxmlformats-officedocument.drawingml.chartshapes+xml"
    )
    DML_DIAGRAM_COLORS = (
        "application/vnd.openxmlformats-officedocument.drawingml.diagramColors+xml"
    )
    DML_DIAGRAM_DATA = (
        "application/vnd.openxmlformats-officedocument.drawingml.diagramData+xml"
    )
    DML_DIAGRAM_DRAWING = "application/vnd.ms-office.drawingml.diagramDrawing+xml"
    DML_DIAGRAM_LAYOUT = (
        "application/vnd.openxmlformats-officedocument.drawingml.diagramLayout+xml"
    )
    DML_DIAGRAM_STYLE = (
        "application/vnd.openxmlformats-officedocument.drawingml.diagramStyle+xml"
    )
    GIF = "image/gif"
    INK = "application/inkml+xml"
    JPEG = "image/jpeg"
    MOV = "video/quicktime"
    MP4 = "video/mp4"
    MPG = "video/mpeg"
    MS_PHOTO = "image/vnd.ms-photo"
    MS_VIDEO = "video/msvideo"
    OFC_CHART_COLORS = "application/vnd.ms-office.chartcolorstyle+xml"
    OFC_CHART_EX = "application/vnd.ms-office.chartex+xml"
    OFC_CHART_STYLE = "application/vnd.ms-office.chartstyle+xml"
    OFC_CUSTOM_PROPERTIES = (
        "application/vnd.openxmlformats-officedocument.custom-properties+xml"
    )
    OFC_CUSTOM_XML_PROPERTIES = (
        "application/vnd.openxmlformats-officedocument.customXmlProperties+xml"
    )
    OFC_DRAWING = "application/vnd.openxmlformats-officedocument.drawing+xml"
    OFC_EXTENDED_PROPERTIES = (
        "application/vnd.openxmlformats-officedocument.extended-properties+xml"
    )
    OFC_OLE_OBJECT = "application/vnd.openxmlformats-officedocument.oleObject"
    OFC_PACKAGE = "application/vnd.openxmlformats-officedocument.package"
    OFC_THEME = "application/vnd.openxmlformats-officedocument.theme+xml"
    OFC_THEME_OVERRIDE = (
        "application/vnd.openxmlformats-officedocument.themeOverride+xml"
    )
    OFC_VML_DRAWING = "application/vnd.openxmlformats-officedocument.vmlDrawing"
    OPC_CORE_PROPERTIES = "application/vnd.openxmlformats-package.core-properties+xml"
    OPC_DIGITAL_SIGNATURE_CERTIFICATE = (
        "application/vnd.openxmlformats-package.digital-signature-certificate"
    )
    OPC_DIGITAL_SIGNATURE_ORIGIN = (
        "application/vnd.openxmlformats-package.digital-signature-origin"
    )
    OPC_DIGITAL_SIGNATURE_XMLSIGNATURE = (
        "application/vnd.openxmlformats-package.digital-signature-xmlsignature+xml"
    )
    OPC_RELATIONSHIPS = "application/vnd.openxmlformats-package.relationships+xml"
    PML_COMMENTS = (
        "application/vnd.openxmlformats-officedocument.presentationml.comments+xml"
    )
    PML_COMMENT_AUTHORS = (
        "application/vnd.openxmlformats-officedocument.presentationml.commen"
        "tAuthors+xml"
    )
    PML_HANDOUT_MASTER = (
        "application/vnd.openxmlformats-officedocument.presentationml.handou"
        "tMaster+xml"
    )
    PML_NOTES_MASTER = (
        "application/vnd.openxmlformats-officedocument.presentationml.notesM"
        "aster+xml"
    )
    PML_NOTES_SLIDE = (
        "application/vnd.openxmlformats-officedocument.presentationml.notesSlide+xml"
    )
    PML_PRESENTATION = (
        "application/vnd.openxmlformats-officedocument.presentationml.presentation"
    )
    PML_PRESENTATION_MAIN = (
        "application/vnd.openxmlformats-officedocument.presentationml.presentation.ma"
        "in+xml"
    )
    PML_PRES_MACRO_MAIN = (
        "application/vnd.ms-powerpoint.presentation.macroEnabled.main+xml"
    )
    PML_PRES_PROPS = (
        "application/vnd.openxmlformats-officedocument.presentationml.presProps+xml"
    )
    PML_PRINTER_SETTINGS = (
        "application/vnd.openxmlformats-officedocument.presentationml.printerSettings"
    )
    PML_SLIDE = "application/vnd.openxmlformats-officedocument.presentationml.slide+xml"
    PML_SLIDESHOW_MAIN = (
        "application/vnd.openxmlformats-officedocument.presentationml.slideshow.main+"
        "xml"
    )
    PML_SLIDE_LAYOUT = (
        "application/vnd.openxmlformats-officedocument.presentationml.slideLayout+xml"
    )
    PML_SLIDE_MASTER = (
        "application/vnd.openxmlformats-officedocument.presentationml.slideMaster+xml"
    )
    PML_SLIDE_UPDATE_INFO = (
        "application/vnd.openxmlformats-officedocument.presentationml.slideUpdateInfo"
        "+xml"
    )
    PML_TABLE_STYLES = (
        "application/vnd.openxmlformats-officedocument.presentationml.tableStyles+xml"
    )
    PML_TAGS = "application/vnd.openxmlformats-officedocument.presentationml.tags+xml"
    PML_TEMPLATE_MAIN = (
        "application/vnd.openxmlformats-officedocument.presentationml.template.main+x"
        "ml"
    )
    PML_VIEW_PROPS = (
        "application/vnd.openxmlformats-officedocument.presentationml.viewProps+xml"
    )
    PNG = "image/png"
    SML_CALC_CHAIN = (
        "application/vnd.openxmlformats-officedocument.spreadsheetml.calcChain+xml"
    )
    SML_CHARTSHEET = (
        "application/vnd.openxmlformats-officedocument.spreadsheetml.chartsheet+xml"
    )
    SML_COMMENTS = (
        "application/vnd.openxmlformats-officedocument.spreadsheetml.comments+xml"
    )
    SML_CONNECTIONS = (
        "application/vnd.openxmlformats-officedocument.spreadsheetml.connections+xml"
    )
    SML_CUSTOM_PROPERTY = (
        "application/vnd.openxmlformats-officedocument.spreadsheetml.customProperty"
    )
    SML_DIALOGSHEET = (
        "application/vnd.openxmlformats-officedocument.spreadsheetml.dialogsheet+xml"
    )
    SML_EXTERNAL_LINK = (
        "application/vnd.openxmlformats-officedocument.spreadsheetml.externalLink+xml"
    )
    SML_PIVOT_CACHE_DEFINITION = (
        "application/vnd.openxmlformats-officedocument.spreadsheetml.pivotCacheDefini"
        "tion+xml"
    )
    SML_PIVOT_CACHE_RECORDS = (
        "application/vnd.openxmlformats-officedocument.spreadsheetml.pivotCacheRecord"
        "s+xml"
    )
    SML_PIVOT_TABLE = (
        "application/vnd.openxmlformats-officedocument.spreadsheetml.pivotTable+xml"
    )
    SML_PRINTER_SETTINGS = (
        "application/vnd.openxmlformats-officedocument.spreadsheetml.printerSettings"
    )
    SML_QUERY_TABLE = (
        "application/vnd.openxmlformats-officedocument.spreadsheetml.queryTable+xml"
    )
    SML_REVISION_HEADERS = (
        "application/vnd.openxmlformats-officedocument.spreadsheetml.revisionHeaders+"
        "xml"
    )
    SML_REVISION_LOG = (
        "application/vnd.openxmlformats-officedocument.spreadsheetml.revisionLog+xml"
    )
    SML_SHARED_STRINGS = (
        "application/vnd.openxmlformats-officedocument.spreadsheetml.sharedStrings+xml"
    )
    SML_SHEET = "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet"
    SML_SHEET_MAIN = (
        "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet.main+xml"
    )
    SML_SHEET_METADATA = (
        "application/vnd.openxmlformats-officedocument.spreadsheetml.sheetMetadata+xml"
    )
    SML_STYLES = (
        "application/vnd.openxmlformats-officedocument.spreadsheetml.styles+xml"
    )
    SML_TABLE = "application/vnd.openxmlformats-officedocument.spreadsheetml.table+xml"
    SML_TABLE_SINGLE_CELLS = (
        "application/vnd.openxmlformats-officedocument.spreadsheetml.tableSingleCells"
        "+xml"
    )
    SML_TEMPLATE_MAIN = (
        "application/vnd.openxmlformats-officedocument.spreadsheetml.template.main+xml"
    )
    SML_USER_NAMES = (
        "application/vnd.openxmlformats-officedocument.spreadsheetml.userNames+xml"
    )
    SML_VOLATILE_DEPENDENCIES = (
        "application/vnd.openxmlformats-officedocument.spreadsheetml.volatileDependen"
        "cies+xml"
    )
    SML_WORKSHEET = (
        "application/vnd.openxmlformats-officedocument.spreadsheetml.worksheet+xml"
    )
    SWF = "application/x-shockwave-flash"
    TIFF = "image/tiff"
    VIDEO = "video/unknown"
    WML_COMMENTS = (
        "application/vnd.openxmlformats-officedocument.wordprocessingml.comments+xml"
    )
    WML_DOCUMENT = (
        "application/vnd.openxmlformats-officedocument.wordprocessingml.document"
    )
    WML_DOCUMENT_GLOSSARY = (
        "application/vnd.openxmlformats-officedocument.wordprocessingml.document.glos"
        "sary+xml"
    )
    WML_DOCUMENT_MAIN = (
        "application/vnd.openxmlformats-officedocument.wordprocessingml.document.main"
        "+xml"
    )
    WML_ENDNOTES = (
        "application/vnd.openxmlformats-officedocument.wordprocessingml.endnotes+xml"
    )
    WML_FONT_TABLE = (
        "application/vnd.openxmlformats-officedocument.wordprocessingml.fontTable+xml"
    )
    WML_FOOTER = (
        "application/vnd.openxmlformats-officedocument.wordprocessingml.footer+xml"
    )
    WML_FOOTNOTES = (
        "application/vnd.openxmlformats-officedocument.wordprocessingml.foot"
        "notes+xml"
    )
    WML_HEADER = (
        "application/vnd.openxmlformats-officedocument.wordprocessingml.header+xml"
    )
    WML_NUMBERING = (
        "application/vnd.openxmlformats-officedocument.wordprocessingml.numbering+xml"
    )
    WML_PRINTER_SETTINGS = (
        "application/vnd.openxmlformats-officedocument.wordprocessingml.printerSettin"
        "gs"
    )
    WML_SETTINGS = (
        "application/vnd.openxmlformats-officedocument.wordprocessingml.settings+xml"
    )
    WML_STYLES = (
        "application/vnd.openxmlformats-officedocument.wordprocessingml.styles+xml"
    )
    WML_WEB_SETTINGS = (
        "application/vnd.openxmlformats-officedocument.wordprocessingml.webSettings+x"
        "ml"
    )
    WMV = "video/x-ms-wmv"
    XML = "application/xml"
    X_EMF = "image/x-emf"
    X_FONTDATA = "application/x-fontdata"
    X_FONT_TTF = "application/x-font-ttf"
    X_MS_VIDEO = "video/x-msvideo"
    X_WMF = "image/x-wmf"


class NAMESPACE(object):
    """Constant values for OPC XML namespaces"""

    DML_WORDPROCESSING_DRAWING = (
        "http://schemas.openxmlformats.org/drawingml/2006/wordprocessingDrawing"
    )
    OFC_RELATIONSHIPS = (
        "http://schemas.openxmlformats.org/officeDocument/2006/relationships"
    )
    OPC_RELATIONSHIPS = "http://schemas.openxmlformats.org/package/2006/relationships"
    OPC_CONTENT_TYPES = "http://schemas.openxmlformats.org/package/2006/content-types"
    WML_MAIN = "http://schemas.openxmlformats.org/wordprocessingml/2006/main"


class RELATIONSHIP_TARGET_MODE(object):
    """Open XML relationship target modes"""

    EXTERNAL = "External"
    INTERNAL = "Internal"


class RELATIONSHIP_TYPE(object):
    AUDIO = "http://schemas.openxmlformats.org/officeDocument/2006/relationships/audio"
    A_F_CHUNK = (
        "http://schemas.openxmlformats.org/officeDocument/2006/relationships/aFChunk"
    )
    CALC_CHAIN = (
        "http://schemas.openxmlformats.org/officeDocument/2006/relationships/calcChain"
    )
    CERTIFICATE = (
        "http://schemas.openxmlformats.org/package/2006/relationships/digital-signatu"
        "re/certificate"
    )
    CHART = "http://schemas.openxmlformats.org/officeDocument/2006/relationships/chart"
    CHARTSHEET = (
        "http://schemas.openxmlformats.org/officeDocument/2006/relationships/chartsheet"
    )
    CHART_COLOR_STYLE = (
        "http://schemas.microsoft.com/office/2011/relationships/chartColorStyle"
    )
    CHART_USER_SHAPES = (
        "http://schemas.openxmlformats.org/officeDocument/2006/relationships/chartUse"
        "rShapes"
    )
    COMMENTS = (
        "http://schemas.openxmlformats.org/officeDocument/2006/relationships/comments"
    )
    COMMENT_AUTHORS = (
        "http://schemas.openxmlformats.org/officeDocument/2006/relationships/commentA"
        "uthors"
    )
    CONNECTIONS = (
        "http://schemas.openxmlformats.org/officeDocument/2006/relationships/connecti"
        "ons"
    )
    CONTROL = (
        "http://schemas.openxmlformats.org/officeDocument/2006/relationships/control"
    )
    CORE_PROPERTIES = (
        "http://schemas.openxmlformats.org/package/2006/relationships/metadata/core-p"
        "roperties"
    )
    CUSTOM_PROPERTIES = (
        "http://schemas.openxmlformats.org/officeDocument/2006/relationships/custom-p"
        "roperties"
    )
    CUSTOM_PROPERTY = (
        "http://schemas.openxmlformats.org/officeDocument/2006/relationships"
        "/customProperty"
    )
    CUSTOM_XML = (
        "http://schemas.openxmlformats.org/officeDocument/2006/relationships/customXml"
    )
    CUSTOM_XML_PROPS = (
        "http://schemas.openxmlformats.org/officeDocument/2006/relationships/customXm"
        "lProps"
    )
    DIAGRAM_COLORS = (
        "http://schemas.openxmlformats.org/officeDocument/2006/relationships/diagramC"
        "olors"
    )
    DIAGRAM_DATA = (
        "http://schemas.openxmlformats.org/officeDocument/2006/relationships/diagramD"
        "ata"
    )
    DIAGRAM_LAYOUT = (
        "http://schemas.openxmlformats.org/officeDocument/2006/relationships/diagramL"
        "ayout"
    )
    DIAGRAM_QUICK_STYLE = (
        "http://schemas.openxmlformats.org/officeDocument/2006/relationships/diagramQ"
        "uickStyle"
    )
    DIALOGSHEET = (
        "http://schemas.openxmlformats.org/officeDocument/2006/relationships/dialogsh"
        "eet"
    )
    DRAWING = (
        "http://schemas.openxmlformats.org/officeDocument/2006/relationships/drawing"
    )
    ENDNOTES = (
        "http://schemas.openxmlformats.org/officeDocument/2006/relationships/endnotes"
    )
    EXTENDED_PROPERTIES = (
        "http://schemas.openxmlformats.org/officeDocument/2006/relationships/extended"
        "-properties"
    )
    EXTERNAL_LINK = (
        "http://schemas.openxmlformats.org/officeDocument/2006/relationships/external"
        "Link"
    )
    FONT = "http://schemas.openxmlformats.org/officeDocument/2006/relationships/font"
    FONT_TABLE = (
        "http://schemas.openxmlformats.org/officeDocument/2006/relationships/fontTable"
    )
    FOOTER = (
        "http://schemas.openxmlformats.org/officeDocument/2006/relationships/footer"
    )
    FOOTNOTES = (
        "http://schemas.openxmlformats.org/officeDocument/2006/relationships/footnotes"
    )
    GLOSSARY_DOCUMENT = (
        "http://schemas.openxmlformats.org/officeDocument/2006/relationships/glossary"
        "Document"
    )
    HANDOUT_MASTER = (
        "http://schemas.openxmlformats.org/officeDocument/2006/relationships/handoutM"
        "aster"
    )
    HEADER = (
        "http://schemas.openxmlformats.org/officeDocument/2006/relationships/header"
    )
    HYPERLINK = (
        "http://schemas.openxmlformats.org/officeDocument/2006/relationships/hyperlin"
        "k"
    )
    IMAGE = "http://schemas.openxmlformats.org/officeDocument/2006/relationships/image"
    MEDIA = "http://schemas.microsoft.com/office/2007/relationships/media"
    NOTES_MASTER = (
        "http://schemas.openxmlformats.org/officeDocument/2006/relationships/notesMas"
        "ter"
    )
    NOTES_SLIDE = (
        "http://schemas.openxmlformats.org/officeDocument/2006/relationships/notesSli"
        "de"
    )
    NUMBERING = (
        "http://schemas.openxmlformats.org/officeDocument/2006/relationships/numberin"
        "g"
    )
    OFFICE_DOCUMENT = (
        "http://schemas.openxmlformats.org/officeDocument/2006/relationships/officeDo"
        "cument"
    )
    OLE_OBJECT = (
        "http://schemas.openxmlformats.org/officeDocument/2006/relationships/oleObjec"
        "t"
    )
    ORIGIN = (
        "http://schemas.openxmlformats.org/package/2006/relationships/digital-signatu"
        "re/origin"
    )
    PACKAGE = (
        "http://schemas.openxmlformats.org/officeDocument/2006/relationships/package"
    )
    PIVOT_CACHE_DEFINITION = (
        "http://schemas.openxmlformats.org/officeDocument/2006/relationships/pivotCac"
        "heDefinition"
    )
    PIVOT_CACHE_RECORDS = (
        "http://schemas.openxmlformats.org/officeDocument/2006/relationships/spreadsh"
        "eetml/pivotCacheRecords"
    )
    PIVOT_TABLE = (
        "http://schemas.openxmlformats.org/officeDocument/2006/relationships/pivotTab"
        "le"
    )
    PRES_PROPS = (
        "http://schemas.openxmlformats.org/officeDocument/2006/relationships/presProp"
        "s"
    )
    PRINTER_SETTINGS = (
        "http://schemas.openxmlformats.org/officeDocument/2006/relationships/printerS"
        "ettings"
    )
    QUERY_TABLE = (
        "http://schemas.openxmlformats.org/officeDocument/2006/relationships/queryTab"
        "le"
    )
    REVISION_HEADERS = (
        "http://schemas.openxmlformats.org/officeDocument/2006/relationships/revision"
        "Headers"
    )
    REVISION_LOG = (
        "http://schemas.openxmlformats.org/officeDocument/2006/relationships/revision"
        "Log"
    )
    SETTINGS = (
        "http://schemas.openxmlformats.org/officeDocument/2006/relationships/settings"
    )
    SHARED_STRINGS = (
        "http://schemas.openxmlformats.org/officeDocument/2006/relationships/sharedSt"
        "rings"
    )
    SHEET_METADATA = (
        "http://schemas.openxmlformats.org/officeDocument/2006/relationships/sheetMet"
        "adata"
    )
    SIGNATURE = (
        "http://schemas.openxmlformats.org/package/2006/relationships/digital-signatu"
        "re/signature"
    )
    SLIDE = "http://schemas.openxmlformats.org/officeDocument/2006/relationships/slide"
    SLIDE_LAYOUT = (
        "http://schemas.openxmlformats.org/officeDocument/2006/relationships/slideLay"
        "out"
    )
    SLIDE_MASTER = (
        "http://schemas.openxmlformats.org/officeDocument/2006/relationships/slideMas"
        "ter"
    )
    SLIDE_UPDATE_INFO = (
        "http://schemas.openxmlformats.org/officeDocument/2006/relationships/slideUpd"
        "ateInfo"
    )
    STYLES = (
        "http://schemas.openxmlformats.org/officeDocument/2006/relationships/styles"
    )
    TABLE = "http://schemas.openxmlformats.org/officeDocument/2006/relationships/table"
    TABLE_SINGLE_CELLS = (
        "http://schemas.openxmlformats.org/officeDocument/2006/relationships/tableSin"
        "gleCells"
    )
    TABLE_STYLES = (
        "http://schemas.openxmlformats.org/officeDocument/2006/relationships/tableSty"
        "les"
    )
    TAGS = "http://schemas.openxmlformats.org/officeDocument/2006/relationships/tags"
    THEME = "http://schemas.openxmlformats.org/officeDocument/2006/relationships/theme"
    THEME_OVERRIDE = (
        "http://schemas.openxmlformats.org/officeDocument/2006/relationships/themeOve"
        "rride"
    )
    THUMBNAIL = (
        "http://schemas.openxmlformats.org/package/2006/relationships/metadata/thumbn"
        "ail"
    )
    USERNAMES = (
        "http://schemas.openxmlformats.org/officeDocument/2006/relationships/username"
        "s"
    )
    VIDEO = "http://schemas.openxmlformats.org/officeDocument/2006/relationships/video"
    VIEW_PROPS = (
        "http://schemas.openxmlformats.org/officeDocument/2006/relationships/viewProp"
        "s"
    )
    VML_DRAWING = (
        "http://schemas.openxmlformats.org/officeDocument/2006/relationships/vmlDrawi"
        "ng"
    )
    VOLATILE_DEPENDENCIES = (
        "http://schemas.openxmlformats.org/officeDocument/2006/relationships/volatile"
        "Dependencies"
    )
    WEB_SETTINGS = (
        "http://schemas.openxmlformats.org/officeDocument/2006/relationships/webSetti"
        "ngs"
    )
    WORKSHEET_SOURCE = (
        "http://schemas.openxmlformats.org/officeDocument/2006/relationships/workshee"
        "tSource"
    )
    XML_MAPS = (
        "http://schemas.openxmlformats.org/officeDocument/2006/relationships/xmlMaps"
    )



# encoding: utf-8

"""Gherkin step implementations for slide background-related features."""

from __future__ import absolute_import, division, print_function, unicode_literals

from behave import given, then

from pptx import Presentation

from helpers import test_pptx


# given ===================================================


@given("a _Background object having {type} background as background")
def given_a_Background_object_having_type_background(context, type):
    sld_idx = {"no": 0, "a fill": 1, "a style reference": None}[type]
    prs = Presentation(test_pptx("sld-background"))
    slide = prs.slide_masters[0] if sld_idx is None else prs.slides[sld_idx]
    context.background = slide.background


# then ====================================================


@then("background.fill is a FillFormat object")
def then_background_fill_is_a_Fill_object(context):
    cls_name = context.background.fill.__class__.__name__
    assert cls_name == "FillFormat", "background.fill is a %s object" % cls_name



# encoding: utf-8

"""
Temporary stand-in for main oxml module that came across with the
PackageReader transplant. Probably much will get replaced with objects from
the pptx.oxml.core and then this module will either get deleted or only hold
the package related custom element classes.
"""

from __future__ import absolute_import

from lxml import etree

from .constants import NAMESPACE as NS, RELATIONSHIP_TARGET_MODE as RTM
from ..oxml import parse_xml, register_element_cls
from ..oxml.simpletypes import (
    ST_ContentType,
    ST_Extension,
    ST_TargetMode,
    XsdAnyUri,
    XsdId,
)
from ..oxml.xmlchemy import (
    BaseOxmlElement,
    OptionalAttribute,
    RequiredAttribute,
    ZeroOrMore,
)


nsmap = {
    "ct": NS.OPC_CONTENT_TYPES,
    "pr": NS.OPC_RELATIONSHIPS,
    "r": NS.OFC_RELATIONSHIPS,
}


def oxml_tostring(elm, encoding=None, pretty_print=False, standalone=None):
    return etree.tostring(
        elm, encoding=encoding, pretty_print=pretty_print, standalone=standalone
    )


def serialize_part_xml(part_elm):
    xml = etree.tostring(part_elm, encoding="UTF-8", standalone=True)
    return xml


class CT_Default(BaseOxmlElement):
    """
    ``<Default>`` element, specifying the default content type to be applied
    to a part with the specified extension.
    """

    extension = RequiredAttribute("Extension", ST_Extension)
    contentType = RequiredAttribute("ContentType", ST_ContentType)


class CT_Override(BaseOxmlElement):
    """
    ``<Override>`` element, specifying the content type to be applied for a
    part with the specified partname.
    """

    partName = RequiredAttribute("PartName", XsdAnyUri)
    contentType = RequiredAttribute("ContentType", ST_ContentType)


class CT_Relationship(BaseOxmlElement):
    """
    ``<Relationship>`` element, representing a single relationship from a
    source to a target part.
    """

    rId = RequiredAttribute("Id", XsdId)
    reltype = RequiredAttribute("Type", XsdAnyUri)
    target_ref = RequiredAttribute("Target", XsdAnyUri)
    targetMode = OptionalAttribute("TargetMode", ST_TargetMode, default=RTM.INTERNAL)

    @classmethod
    def new(cls, rId, reltype, target, target_mode=RTM.INTERNAL):
        """
        Return a new ``<Relationship>`` element.
        """
        xml = '<Relationship xmlns="%s"/>' % nsmap["pr"]
        relationship = parse_xml(xml)
        relationship.rId = rId
        relationship.reltype = reltype
        relationship.target_ref = target
        relationship.targetMode = target_mode
        return relationship


class CT_Relationships(BaseOxmlElement):
    """
    ``<Relationships>`` element, the root element in a .rels file.
    """

    relationship = ZeroOrMore("pr:Relationship")

    def add_rel(self, rId, reltype, target, is_external=False):
        """
        Add a child ``<Relationship>`` element with attributes set according
        to parameter values.
        """
        target_mode = RTM.EXTERNAL if is_external else RTM.INTERNAL
        relationship = CT_Relationship.new(rId, reltype, target, target_mode)
        self._insert_relationship(relationship)

    @classmethod
    def new(cls):
        """
        Return a new ``<Relationships>`` element.
        """
        xml = '<Relationships xmlns="%s"/>' % nsmap["pr"]
        relationships = parse_xml(xml)
        return relationships

    @property
    def xml(self):
        """
        Return XML string for this element, suitable for saving in a .rels
        stream, not pretty printed and with an XML declaration at the top.
        """
        return oxml_tostring(self, encoding="UTF-8", standalone=True)


class CT_Types(BaseOxmlElement):
    """
    ``<Types>`` element, the container element for Default and Override
    elements in [Content_Types].xml.
    """

    default = ZeroOrMore("ct:Default")
    override = ZeroOrMore("ct:Override")

    def add_default(self, ext, content_type):
        """
        Add a child ``<Default>`` element with attributes set to parameter
        values.
        """
        return self._add_default(extension=ext, contentType=content_type)

    def add_override(self, partname, content_type):
        """
        Add a child ``<Override>`` element with attributes set to parameter
        values.
        """
        return self._add_override(partName=partname, contentType=content_type)

    @classmethod
    def new(cls):
        """
        Return a new ``<Types>`` element.
        """
        xml = '<Types xmlns="%s"/>' % nsmap["ct"]
        types = parse_xml(xml)
        return types


register_element_cls("ct:Default", CT_Default)
register_element_cls("ct:Override", CT_Override)
register_element_cls("ct:Types", CT_Types)

register_element_cls("pr:Relationship", CT_Relationship)
register_element_cls("pr:Relationships", CT_Relationships)



# encoding: utf-8

"""
Provides the PackURI value type along with some useful known pack URI strings
such as PACKAGE_URI.
"""

import posixpath
import re


class PackURI(str):
    """
    Provides access to pack URI components such as the baseURI and the
    filename slice. Behaves as |str| otherwise.
    """

    _filename_re = re.compile("([a-zA-Z]+)([0-9][0-9]*)?")

    def __new__(cls, pack_uri_str):
        if not pack_uri_str[0] == "/":
            tmpl = "PackURI must begin with slash, got '%s'"
            raise ValueError(tmpl % pack_uri_str)
        return str.__new__(cls, pack_uri_str)

    @staticmethod
    def from_rel_ref(baseURI, relative_ref):
        """
        Return a |PackURI| instance containing the absolute pack URI formed by
        translating *relative_ref* onto *baseURI*.
        """
        joined_uri = posixpath.join(baseURI, relative_ref)
        abs_uri = posixpath.abspath(joined_uri)
        return PackURI(abs_uri)

    @property
    def baseURI(self):
        """
        The base URI of this pack URI, the directory portion, roughly
        speaking. E.g. ``'/ppt/slides'`` for ``'/ppt/slides/slide1.xml'``.
        For the package pseudo-partname '/', baseURI is '/'.
        """
        return posixpath.split(self)[0]

    @property
    def ext(self):
        """
        The extension portion of this pack URI, e.g. ``'xml'`` for
        ``'/ppt/slides/slide1.xml'``. Note that the period is not included.
        """
        # raw_ext is either empty string or starts with period, e.g. '.xml'
        raw_ext = posixpath.splitext(self)[1]
        return raw_ext[1:] if raw_ext.startswith(".") else raw_ext

    @property
    def filename(self):
        """
        The "filename" portion of this pack URI, e.g. ``'slide1.xml'`` for
        ``'/ppt/slides/slide1.xml'``. For the package pseudo-partname '/',
        filename is ''.
        """
        return posixpath.split(self)[1]

    @property
    def idx(self):
        """
        Return partname index as integer for tuple partname or None for
        singleton partname, e.g. ``21`` for ``'/ppt/slides/slide21.xml'`` and
        |None| for ``'/ppt/presentation.xml'``.
        """
        filename = self.filename
        if not filename:
            return None
        name_part = posixpath.splitext(filename)[0]  # filename w/ext removed
        match = self._filename_re.match(name_part)
        if match is None:
            return None
        if match.group(2):
            return int(match.group(2))
        return None

    @property
    def membername(self):
        """
        The pack URI with the leading slash stripped off, the form used as
        the Zip file membername for the package item. Returns '' for the
        package pseudo-partname '/'.
        """
        return self[1:]

    def relative_ref(self, baseURI):
        """
        Return string containing relative reference to package item from
        *baseURI*. E.g. PackURI('/ppt/slideLayouts/slideLayout1.xml') would
        return '../slideLayouts/slideLayout1.xml' for baseURI '/ppt/slides'.
        """
        # workaround for posixpath bug in 2.6, doesn't generate correct
        # relative path when *start* (second) parameter is root ('/')
        if baseURI == "/":
            relpath = self[1:]
        else:
            relpath = posixpath.relpath(self, baseURI)
        return relpath

    @property
    def rels_uri(self):
        """
        The pack URI of the .rels part corresponding to the current pack URI.
        Only produces sensible output if the pack URI is a partname or the
        package pseudo-partname '/'.
        """
        rels_filename = "%s.rels" % self.filename
        rels_uri_str = posixpath.join(self.baseURI, "_rels", rels_filename)
        return PackURI(rels_uri_str)


PACKAGE_URI = PackURI("/")
CONTENT_TYPES_URI = PackURI("/[Content_Types].xml")
